/******************************* MAIN HEADER **********************************
    Title:              There Be Somethin' Fishy Goin' On ...
    Files:              Aquarium.java, Fish.java
    
    Author:             Data Fok / data@cs.wisc.edu
    Collaborators:      N/A
    
    Due Date:           7:30 AM, Monday, Nov. 6, 2000
    Completion Date:    11/5/2000
    
    Course:             CS 302, Fall 2000, Section 14
    TA:                 Jeff Shergali
    
    Compiler:           CodeWarrior IDE 4.0 (JDK 1.2)
    Platform:           Windows NT 4.0
 **************************** 80 columns wide *********************************/

import java.awt.Color;
import javabook.*;

/** 
 * Runs an aquarium simulation with a user specified number of Fish,
 * Piranha, and FoodCenters.
 * 
 * Bugs: No known bugs. 
 * @author Data Fok / datafok@hotmail.com
 **/ 

class Aquarium {

	private static Tank tank;		// ALLOWS tank TO BE SHARED BY OTHER METHODS

    /**         
     * Display Aquarium Statues i.e. the information about each fish, piranha 
     * and foodCenter. Information include: Location, FoodPoints and Direction
     * 
     **/
    private static void displayStats() {
    	OutputBox oBox = new OutputBox(tank);
       	
    	//displaying fish stats
    	oBox.printLine("Fish");
    	oBox.printLine("Location     Direction     FoodPoints");
    	oBox.printLine("-------------------------------------");
    	FishList fList = tank.getFishList();
    	while ( fList.hasMoreFish() ) {
    		Fish fish = fList.nextFish();
    		Location fishLoc = fish.getLocation();
    		int fishPoint = fish.getFoodPoints();
    		boolean fDir = fish.getDirection();
    		String sFDir;
    		if ( fDir == Tank.LEFT) {
    			sFDir = "LEFT";
    		}
    		else {
    			sFDir = "RIGHT";
    		}
    		oBox.print(Format.leftAlign(13, fishLoc.getX()+","+fishLoc.getY() ));
    		oBox.print(Format.leftAlign(14, sFDir));
    		oBox.printLine(Format.leftAlign(10, fishPoint));
    	}
    	oBox.skipLine(1);
    	
    	//displaying pir stats
    	oBox.printLine("Piranha");
    	oBox.printLine("Location     Direction     FoodPoints");
    	oBox.printLine("-------------------------------------");
    	PiranhaList pList = tank.getPiranhaList();
    	while ( pList.hasMorePiranha() ) {
    		Piranha pir = pList.nextPiranha();
    		Location pirLoc = pir.getLocation();
    		int pirPoint = pir.getFoodPoints();
    		boolean pDir = pir.getDirection();
    		String sPDir;
    		if ( pDir == Tank.LEFT) {
    			sPDir = "LEFT";
    		}
    		else {
    			sPDir = "RIGHT";
    		}
    		oBox.print(Format.leftAlign(13, pirLoc.getX()+","+pirLoc.getY() ));
    		oBox.print(Format.leftAlign(14, sPDir));
    		oBox.printLine(Format.leftAlign(10, pirPoint));
    	}
    	oBox.skipLine(1);
    	
    	//displaying foodcenter stats
    	oBox.printLine("Food Center");
    	oBox.printLine("Location     FoodPoints");
    	oBox.printLine("-----------------------");
    	FoodCentersList foodList = tank.getFoodCentersList();
    	while ( foodList.hasMoreFoodCenters() ) {
    		FoodCenter foodCenter = foodList.nextFoodCenter();
    		Location foodLoc = foodCenter.getLocation();
    		int foodCenterPoint = foodCenter.getFoodPoints();
    		oBox.print(Format.leftAlign(13, foodLoc.getX()+","+foodLoc.getY() ));
    		oBox.printLine(Format.leftAlign(10, foodCenterPoint));
    	}
    	    	
    	oBox.show();
		oBox.waitUntilClose();    
    }

    /**         
     * Show the movements in the tank for a given number of times
     * @param numMove number of movement of the simulation 
     **/
	private static void movement(int numMove) { 
		for ( int i = 1 ; i <= numMove; i++ ) {
			tank.refreshDisplay();
			moveOfPir();
			moveOfFish();			
			tank.refreshDisplay();
		}
	}

	/**         
     * The movement of the piranha. If piranha is hungry and a fish is within
     * a 3x3 grid centered on a piranha, it will be eaten the foodPoints 
     * of the fish will be taken up by the piranha. Provided that the
     * fish is not dead.
     *  
     **/
	private static void moveOfPir() {
		PiranhaList pList = tank.getPiranhaList();
		while ( pList.hasMorePiranha() ) {
			Piranha pir = pList.nextPiranha();
			if ( !(pir.isDead()) ) {
				pir.pirMove();
				FishList fList = tank.getFishList();
				while (fList.hasMoreFish() ) {
					Fish fish = fList.nextFish();
					//check if the fish is within a 3x3 grid centered on a piranha.
					int dis = pir.getLocation().chessBoardDistance(fish.getLocation());
					if ( (dis <= 1) && !(fish.isDead()) && (pir.getColor() == Color.white) 
						&& fish.getColor() == Color.white) {
						pir.eatFish(fish.getFoodPoints());
						fish.eaten(); //fish is dead if eaten.
					}
					else { } 
				}
			}
			else { }
		}
	}
	
	/**         
     * The movement of the fish. If the fish is not full and a foodcenter
     * is at the same location of the fish the foodPoint of the foodCenter
     * will be taken up by the fish. Provided that the foodCenter is not
     * empty. A fish can feed on more than one foodCenter if it is not full.
     *  
     **/
	private static void moveOfFish() {
		FishList fList = tank.getFishList();
		while ( fList.hasMoreFish() ) {
			Fish fish = fList.nextFish();
			if ( !(fish.isDead()) ) {
				fish.fishMove();
				FoodCentersList foodList = tank.getFoodCentersList();
				while (foodList.hasMoreFoodCenters() ) {
					FoodCenter foodCenter = foodList.nextFoodCenter();
					if ( fish.getLocation().equals(foodCenter.getLocation())  && (fish.getFoodPoints() < tank.FISH_MAX_POINTS) && (foodCenter.getFoodPoints() > 0 ) ) {
						fish.eat(foodCenter.consume(tank.FISH_MAX_POINTS - fish.getFoodPoints() ) );
					}
					else { }
				}
			}
			else { }
		}
	}
	
	/**         
     * Check if all fish in the tank are dead
     *  
     * @return true if all the fishes have no foodpoint, i.e. dead
     * @return false if there are still fish alive in the tank
     **/
	private static boolean allFishDead() {
		FishList fList = tank.getFishList();
		while (fList.hasMoreFish() ) {
			Fish fish = fList.nextFish();
			if ( !(fish.isDead()) ) {
				return false ;
			}
			else { }
		}
		return true;
	}
	
	/**         
     * Check if all piranha in the tank are dead
     *  
     * @return true if all the piranha have no foodpoint, i.e. dead
     * @return false if there are still piranha alive in the tank
     **/
	private static boolean allPirDead() {
		PiranhaList pList = tank.getPiranhaList();
		while (pList.hasMorePiranha() ) {
			Piranha pir = pList.nextPiranha();
			if ( !(pir.isDead()) ) {
				return false ;
			}
			else { }
		}
		return true;
	}
	
	/**         
     * As long as there is still fish or piranha alive in the tank,
     * run the simulation
	 *    
     **/
	private static void continuous(){
		tank.refreshDisplay();
		while ( !(allFishDead() && allPirDead()) ) {
			moveOfPir();
			moveOfFish();
			tank.refreshDisplay();	
		}
		System.exit(0); //exit program when all fish and piranha die
	}
    
    /**         
     * The main class of the program.
     * Ask the user for input of number of fish, piranha and foodcenter.
     * Add those object to the tank. Display list box for user to choose
     * how to run the simulation and the status of the aquarium, untill
     * the user select quit or cancel.
     *
     **/
    public static void main(String [] args) {
    
    	int fishNum, pirNum, foodCenNum; //the number of fish, piranha and 
    									 //foodcenter appear	
    	
    	boolean keeplooping = true;
    	
    	tank = new Tank();
        InputBox iBox = new InputBox(tank);
        MessageBox mBox = new MessageBox(tank);
        
        //asking the user for numbers of fish, piranha and foodcenter in the tank
        
        boolean notValid = false;
        
        do {
        	fishNum = iBox.getInteger("Please Enter the number of fish you want: ");
         	if (fishNum < 0) {
         		mBox.show("Invalid input. Number of fish should be greater than 0 !");
         		notValid = true;
         		}
         	else { 
         		notValid = false;
         		}
        } while (notValid);
                 
        
        do {
        	pirNum = iBox.getInteger("Please Enter the number of piranha you want: ");
        	if (pirNum < 0) {
         		mBox.show("Invalid input. Number of piranha should be greater than 0 !");
         		notValid = true;
         		}
         	else { 
         		notValid = false;
         		}
         }  while (notValid);
        
        do {
        	foodCenNum = iBox.getInteger("Please Enter the number of foodcenter you want: ");
        	if (foodCenNum < 0) {
         		mBox.show("Invalid input. Number of foodcenter should be greater than 0 !");
         		notValid = true;
         		}
         	else { 
         		notValid = false;
         		}
         }  while (notValid);
        
        //add objects to tank            
        for ( int i=1; i<=fishNum; i++ ) {
        	Fish fish = new Fish();
        	tank.addFish(fish);
        }
                  
        for	( int j=1; j<=pirNum; j++ ) {
        	Piranha piranha = new Piranha();
        	tank.addPiranha(piranha);
        }
        
        for ( int z=1; z<=foodCenNum; z++ ) {
        	FoodCenter foodCenter = new FoodCenter();
        	tank.addFoodCenter(foodCenter);
        }
                
        //show the menu for the user to select what to do
        ListBox menu = new ListBox(tank, "Menu");
        menu.addItem("Display Aquarium Status");
        menu.addItem("Next Movement");
        menu.addItem("Next N Movement");
        menu.addItem("Continuous Mode");
        menu.addItem("Quit");
                
        while (keeplooping) {
        
        	int selection = menu.getSelectedIndex();
        	switch (selection) {
        	
        		case 0: displayStats();
        				break;
     	   			
        		case 1: movement(1);
        				break;
        			
       		 	case 2: boolean flag = false;
       		 			int n;
       		 			
       		 			do {
       		 				n = iBox.getInteger("Please enter the number of movement to make: ");
        					if (n < 0) {
        						mBox.show("Invalid input. Number of movement should be greater than 0");
        						flag = true;
        						}
        					else {
        						flag = false;
        					}
        				} while (flag);
        				
        				movement(n);
        				break;
        		
     		   	case 3: continuous();
       					break;
       		
       			case 4: mBox.show("Quit selected");
       					System.exit(0);
    					break;
    		
   	 			case ListBox.CANCEL: System.exit(0);
    						 	     break;
    						 	 
    			case ListBox.NO_SELECTION: mBox.show("You selected nothing, please make a selection.");
    								       break;
       	
        	}
    	}
    }
}        
