/******************************* FILE HEADER **********************************
    Main Class File:  Aquarium.java
    File:             Fish.java
    
    Author:           Data Fok / data@cs.wisc.edu
    Collaborators:    N/A
    Completion Date:  11/4/2000
    
    Course:           CS 302, Fall 2000, Section 14
    TA:               Jeff Shergali

    Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
    Platform:         Windows NT 4.0
 **************************** 80 columns wide *********************************/

import java.awt.Color;

/** 
 * Represents a Fish. 
 * The fish will be place randomly at first and set to FISH_MAX_POINTS.
 * It moves around randomly, eats in FoodCenters, being eaten by Piranha
 * It changes color according to its foodPoints and disappear from the tank
 * once its foodPoints drops to zero, ie, dead.
 *
 * Bugs: No known bugs. 
 * @author Data Fok / datafok@hotmail.com
 **/ 

class Fish {

    private int foodPoints ;   // food points this Fish has 
    						   // Range: 0 to Tank.FISH_MAX_POINTS
    private Location location; // where this fish is in the tank
    private int direction;     // heading of the fish, 0:left 1:right
    private Color color;       // color of the fish
        
    /**         
     * Constructs a fish and initializes its data members,
     * such as, location, direction, foodPoints and color
     **/
    public Fish ( ) {
        location = ranLoc();
        direction = ranDir();
        foodPoints = Tank.FISH_MAX_POINTS;
    	color = fishColor();
    }
    
    /**         
     * Constructs a fish and initializes its data members,
     * such as, location, direction, foodPoints and color
     * @param a the x-coordinate of the fish
     * @param b the y-coordinate of the fish
     * @param point the foodPoints of the fish
     * @param d the direction of the fish 0:left 1:right   
     **/
    public Fish (int a, int b, int point, int d) {
        location = new Location(a,b);
        direction = d;
        foodPoints = point;
        color = fishColor();
    }
   
    /**         
  	 * Random a location for the fish when it is created
  	 * 0 <= x-coordinate of the fish <= TANK_WIDTH
  	 * 0 <= y-coordinate of the fish <= WATER_HEIGHT
     **/
    private Location ranLoc() { 
    	int x = (int)Math.floor((Tank.TANK_WIDTH+1) * Math.random());
    	int y = (int)Math.floor((Tank.WATER_HEIGHT+1) * Math.random());
    	return new Location(x,y);
    }
    
    /**         
     * Random a direction for the fish when it is created
     * The fish is pointing either left or right
     **/
    private int ranDir() { 
    	int x = (int)Math.floor( Math.random() * 2);
    	if ( x == 0 ) {  //if x is 0. fish is heading left
    		return 0;
    	}
    	else { //if x is 1, fish is heading right
    		return 1;
    	}
    }
    
    /**         
  	 * Change the color of the fish according to its foodPoints
     * Color of the fish is pink initially, when the foodPoints
     * drops below its strength threshold (weak), it changes to white
     **/
    private Color fishColor() { 
    	if ( foodPoints < (int)(Tank.FISH_MAX_POINTS * Tank.STRENGTH_THRESHOLD) ) {
    		return Color.white; 
    	}
    	else {
    		return Color.pink;
    	}
    }
    
    /**         
     * Switch the direction of the fish if it hits the walls of the tank.
     * @param y the y-coordinate of the fish before movement
     * @return boolean Whether the fish has flipped or not
     **/
    private void fishFlip(int y) { 
    	if (location.getX() > Tank.TANK_WIDTH) {
    		location = new Location(Tank.TANK_WIDTH, y );
    		direction = 0;
    		
    	}
    	else if ( location.getX() < 0) {
    			location = new Location( 0, y );
    			direction = 1;
    		 	
    		 }
    		 else { }

    }
    
    /**         
  	 * Get the location of the fish
     * 
     * @return location (the location of the fish)
     **/
    public Location getLocation() {
        return location;
    }
    
    /**         
     * Get the direction of the fish
     * 
     * @return direction (Tank.LEFT or Tank.RIGHT)
     **/
    public boolean getDirection() {
        if (direction == 0 ) { 
        	return Tank.LEFT;  //0: left
        }
        else {
        	return Tank.RIGHT; //1: right
        }
    }
    
    /**         
     * Get the color of the fish
     * 
     * @return color (white or pink)
     **/
    public Color getColor() {
    	return color;
    }
    
    /**         
     * Get the foodPoint of the fish
     * 
     * @return foodPoints the amount of food the fish have
     **/
    public int getFoodPoints() {
        return foodPoints;
    }
    
    /**         
     * Set foodPoints to zero when eaten by Piranha
     * 
     **/
    public void eaten() {
    	foodPoints = 0;
    }
    
    /**         
     * Check whether the foodPoint is equal or below zero,
     * if so declare it as "Dead" fish
     *
     * @return true if foodPoints <= 0, else false
     **/
    public boolean isDead() {
        if (foodPoints <= 0) {
        	return true;
        }
        else {
        	return false;
        }
    }
    
    /**         
     * As long as the fish is not dead take up foodPoints from foodCenter 
     * if foodPoints is less then FISH_MAX_POINTS. Then check if need to 
     * change color.
     * @param upTaken the amount of foodPoints taken up from foodCenter
     **/
    public void eat(int upTaken) {
    	if ( !( isDead() ) ){ 
	    	if (foodPoints < Tank.FISH_MAX_POINTS) {
    			foodPoints += upTaken;
    		}
    		else { }
    	}
    	else { } 
    	color = fishColor(); 
    }

    /**         
     * Change the location of the fish by the random function from Math Class,
     * so as to "move" the fish. Check if need to flip the fish, then minus 
     * the foodPoints by FISH_MOVE_COST. Then check if need to change color.
     * The fish cannot "stand still", thus dx and dy cannot both equal to zero.
     **/
	public void fishMove() {
		int ranX = 0; // the movement in x direction
		int ranY = 0; // the movement in y direction
				
		
			//assume fish is heading left
			if ((location.getY() == Tank.WATER_HEIGHT) || (location.getY() == 0)) {
			//when the fish is at the top (0) or at the bottom (Tank.WATER_HEIGHT)
				if (location.getY() == Tank.WATER_HEIGHT) { //at the bottom
					//the range of ranX is -1 <= dx <= 0
					//the range of ranY is -1 <= dy <= 0
					do {
						ranX = (int)Math.floor( 2 * Math.random() ) - 1;
						ranY = (int)Math.floor( 2 * Math.random() ) - 1;
					}while ( ranX == 0 && ranY == 0);  
				}
				else { 	// at the top
					//the range of ranX is -1 <= dx <= 0
					//the range of ranY is 0 <= dy <= 1
					do {
						ranX = (int)Math.floor( 2 * Math.random() ) - 1;
						ranY = (int)Math.floor( 2 * Math.random() );
					}while (ranX == 0 && ranY ==0);
				}
			}
			else {
			//when the fish locates in between the top and the bottom of the tank
			//the range of ranX is -1 <= dx <= 0
			//the range of ranY is -1 <= dy <= 1
				do {
					ranX = (int)Math.floor( 2 * Math.random() ) - 1;
					ranY = (int)Math.floor( 3 * Math.random() ) - 1;
				}while (ranX == 0 && ranY ==0);
			}
		
			if ( direction == 1) { //consider when fish heading right
				ranX *= (-1);      //all the movement is x is opposite
			}
			else { }
		
		int oriY = location.getY();
		location.move(ranX,ranY); //move the fish to a new location
		fishFlip(oriY);		
		foodPoints -= Tank.FISH_MOVE_COST; 
		color = fishColor(); //check if need to change color
	}
}
