/******************************* FILE HEADER **********************************
    Main Class File:  Aquarium.java
    File:             FoodCenter.java
    
    Author:           Data Fok / data@cs.wisc.edu
    Collaborators:    N/A
    Completion Date:  11/5/2000
    
    Course:           CS 302, Fall 2000, Section 14
    TA:               Jeff Shergali

    Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
    Platform:         Windows NT 4.0
 **************************** 80 columns wide *********************************/

import java.awt.Color;

/** 
 * Represents a FoodCenter. 
 * The foodcenter will be place randomly at first and set to FOOD_CENTER_POINTS.
 * Fish will take up foodPoints in the foodCenter. The color of the foodCenter
 * will change according to the foodPoints left in it. 
 *
 * Bugs: No known bugs. 
 **/ 

class FoodCenter {

    private int foodPoints ;   // food points this FoodCenter has  
                               // Range: 0 to Tank.FOOD_CENTER_POINTS
    private Location location; // where this foodcenter is in the tank
    private Color color;       // color of the fish
        
    /**         
     * Constructs a foodCenter and initializes its data members,
     * such as, location, foodPoints and color
     **/
    public FoodCenter ( ) {
        location = ranLoc();
        foodPoints = Tank.FOOD_CENTER_POINTS;
    	color = foodColor();
    }
    
    /**         
     * Constructs a foodCenter and initializes its data members,
     * such as, location, direction, foodPoints and color
     * @param a the x-coordinate of the foodCenter
     * @param b the y-coordinate of the foodCenter
     * @param point the foodPoints of the foodCenter
     **/
    public FoodCenter (int a, int b, int point) {
        location = new Location(a, b);
        foodPoints = point;
        color = foodColor();
    }
       
    /**         
  	 * Random a location for the foodCenter when it is created
  	 * 0 <= x-coordinate of the foodCenter <= TANK_WIDTH
  	 * 0 <= y-coordinate of the foodCenter <= WATER_HEIGHT
     **/
    private Location ranLoc() { //random the location of the foodcenter
    	int x = (int)Math.floor((Tank.TANK_WIDTH+1) * Math.random());
    	int y = (int)Math.floor((Tank.WATER_HEIGHT+1) * Math.random());
    	return new Location(x,y);
    }
        
    /**         
  	 * Change the color of the foodCenter according to its foodPoints
     * Color of the foodCenter is green when full. If the foodPoints
     * drops to half of the max. value, it changes to yellow. When 
     * foodPoints equals to zero, it changes to red.
     **/
    private Color foodColor() { 
    	if ( foodPoints == 0 ) { 
    		return Color.red; 
    	}
    	else if ( foodPoints < (Tank.FOOD_CENTER_POINTS/2) ) {		
    			return Color.yellow; 
    		}
    		else {
    			return Color.green;
    		}
    }
    
    /**         
     * Minus the foodPoints taken up by the fish. When the food need 
     * by the fish is more than the remaining foodPoints, the fish
     * will only take up what remain.Then Check if need to change color.
     * 
     * @return how much will be taken up by the fish
     **/
    public int consume(int foodNeed) { 
    	if ( foodPoints == 0 ) {	   
    		color = foodColor();
    		return 0;
    	}
    	else if ( foodNeed >= foodPoints) {        
    			int tempFoodPoints = foodPoints;   
    			foodPoints = 0;					   
    			color = foodColor();
    			return tempFoodPoints;
    		 }
    		 else {
    			foodPoints -= foodNeed;
    			color = foodColor();
    			return foodNeed;
    		 }
    }    	
    
    /**         
  	 * Get the location of the foodCenter
     * 
     * @return location (the location of the foodCenter)
     **/
    public Location getLocation() {
        return location;
    }
    
    /**         
     * Get the color of the foodCenter
     * 
     * @return color (green, yellow or red)
     **/    
    public Color getColor() {
    	return color;
    }
    
    /**         
     * Get the foodPoint of the foodCenter
     * 
     * @return foodPoints the amount of food the foodCenter have
     **/
    public int getFoodPoints() {
        return foodPoints;
    }           
}