/******************************* FILE HEADER **********************************
    Main Class File:  Aquarium.java
    File:             Piranha.java
    
    Author:           Data Fok / data@cs.wisc.edu
    Collaborators:    N/A
    Completion Date:  11/5/2000
    
    Course:           CS 302, Fall 2000, Section 14
    TA:               Jeff Shergali

    Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
    Platform:         Windows NT 4.0
 **************************** 80 columns wide *********************************/

import java.awt.Color;

/** 
 * Represents a Piranha. 
 * The piranha will be place randomly at first and set to PIRANHA_MAX_POINTS.
 * It moves around randomly, feeds on fish. It changes color according to 
 * its foodPoints and disappear from the tank once its foodPoints drops to zero.
 * ie, dead.
 *
 * Bugs: No known bugs. 
 * @author Data Fok / datafok@hotmail.com
 **/

class Piranha {

    private int foodPoints ;   // food points this Piranha has  
                               // Range: 0 to Tank.PIRANHA_MAX_POINTS
    private Location location; // where this piranha is in the tank
    private int direction; // heading of the piranha
    private Color color;   //color of the piranha
    
    /**         
     * Constructs a piranha and initializes its data members
     * such as, location, direction, foodPoints and color
     **/
    public Piranha ( ) {
        location = ranLoc();
        direction = ranDir();
        foodPoints = Tank.PIRANHA_MAX_POINTS;
        color = pirColor();
    }
    
    /**         
     * Constructs a piranha and initializes its data members
     * such as, location, direction, foodPoints and color
     * @param a the x-coordinate of the piranha
     * @param b the y-coordinate of the piranha
     * @param point the foodPoints of the piranha
     * @param d the direction of the piranha 0:left 1:right   
     **/
    public Piranha (int a, int b ,int point, int d) {
        location = new Location(a,b);
        direction = d;
        foodPoints = point;
        color = pirColor();
    }
    
    /**         
  	 * Random a location for the piranha when it is created
  	 * 0 <= x-coordinate of the piranha <= TANK_WIDTH
  	 * 0 <= y-coordinate of the piranha <= WATER_HEIGHT
     **/
    private Location ranLoc() { 
    	int x = (int)Math.floor((Tank.TANK_WIDTH+1) * Math.random());
    	int y = (int)Math.floor((Tank.WATER_HEIGHT+1) * Math.random());
    	return new Location(x,y);
    }
    
    /**         
     * Random a direction for the piranha when it is created
     * The piranha is pointing either left or right
     **/
    private int ranDir() {
    	int x = (int)Math.floor( Math.random() * 2);
    	if ( x == 0 ) {  //if x is 0. piranha is heading left
    		return 0;
    	}
    	else { //if x is 1, piranha is heading right
    		return 1;
    	}
    }
    
    /**         
  	 * Change the color of the piranha according to its foodPoints
     * Color of the piranha is gray initially, when the foodPoints
     * drops below its hunger threshold (weak), it changes to white
     **/
    private Color pirColor() { 
    	if (foodPoints < (int)(Tank.PIRANHA_MAX_POINTS * Tank.HUNGER_THRESHOLD)) {
    		return Color.white; 
    	}
    	else {
    		return Color.gray;
    	}
    }
    
    /**         
     * Switch the direction of the piranha if it hits the walls of the tank.
     * @param y the y-coordinate of the piranha before movement
     * @return boolean Whether the fish has flipped or not     
     **/
    private void pirFlip(int y) { 


    	if (location.getX() > Tank.TANK_WIDTH) {
    		location = new Location(Tank.TANK_WIDTH, y );
    		direction = 0;
    		
    	}
    	else if ( location.getX() < 0) {
    			location = new Location( 0, y );
    			direction = 1;
    	
    		 }
    		 else { }

    }
    
    /**         
  	 * Get the location of the piranha
     * 
     * @return location (the location of the piranha)
     **/
    public Location getLocation() {
        return location;
    }
    
    /**         
     * Get the direction of the piranha
     * 
     * @return direction (Tank.LEFT or Tank.RIGHT)
     **/
    public boolean getDirection() {
       if (direction == 0 ) {
        	return Tank.LEFT;  //0: left
        }
        else {
        	return Tank.RIGHT; //1: right
        }
    }
    
    /**         
     * Get the color of the piranha
     * 
     * @return color (white or pink)
     **/
    public Color getColor() {
    	return color;
    }
    
    /**         
     * Get the foodPoint of the piranha
     * 
     * @return foodPoints the amount of food the piranha have
     **/
    public int getFoodPoints() {
        return foodPoints;
    }
    
    /**         
     * Check whether the foodPoint is equal or below zero,
     * if so declare it as "Dead" piranha
     *
     * @return true if foodPoints <= 0, else false
     **/
    public boolean isDead() {
        if (foodPoints <= 0) {
        	return true;
        }
        else {
        	return false;
        }
    }
    
    /**         
     * As long as the piranha is not dead take up foodPoints from fish 
     * if foodPoints is below the percentage of the HUNGER_THRESHOLD.
     * Then check if need to change color.
     * @param upTaken the amount of foodPoints taken up from fish
     **/
    public void eatFish(int upTaken) {
    	if ( !( isDead() ) ){
	    	if (foodPoints < (Tank.PIRANHA_MAX_POINTS*Tank.HUNGER_THRESHOLD)) {
    			foodPoints += upTaken;
    		}
    		else { }
    	}
    	else { } 
    	color = pirColor();
    }
    
    /**         
     * Change the location of the piranha by generating x and y movement
     * from the random function from Math Class, so as to "move" the piranha. 
     * After that check if need to flip the piranha, then minus the foodPoints 
     * by PIRANHA_MOVE_COST. Then check if need to change color.
     * The piranha cannot "stand still", thus dx and dy cannot both equal to
     * zero.
     **/
    public void pirMove() {
		int ranX = 0; // the movement in x direction
		int ranY = 0; // the movement in y direction
				
		
			//assume the piranha is at left
			if ((location.getY() == Tank.WATER_HEIGHT) || (location.getY() == 0)) {
			//when the piranha is at the top (0) or at the bottom (Tank.WATER_HEIGHT)
				if (location.getY() == Tank.WATER_HEIGHT) { //at the bottom
					//the range of dx is -1 <= dx <= 0
					//the range of dy is -1 <= dy <= 0
					do {
						ranX = (int)Math.floor( 2 * Math.random() ) - 1;
						ranY = (int)Math.floor( 2 * Math.random() ) - 1;
					}while ( ranX == 0 && ranY == 0);
				}
				else { 	// at the top
					//the range of dx is -1 <= dx <= 0
					//the range of dy is 0 <= dy <= 1
					do {
						ranX = (int)Math.floor( 2 * Math.random() ) - 1;
						ranY = (int)Math.floor( 2 * Math.random() );
					}while (ranX == 0 && ranY ==0);
				}
			}
			else {
			//when the piranha locates in between the top and the bottom of the tank
			//the range of dx is -1 <= dx <= 0
			//the range of dy is -1 <= dy <= 1
				do {
					ranX = (int)Math.floor( 2 * Math.random() ) - 1;
					ranY = (int)Math.floor( 3 * Math.random() ) - 1;
				}while (ranX == 0 && ranY ==0);
			}
		
			if ( direction == 1) { //consider when piranha heading right
				ranX *= (-1);      //all the movement is x is opposite
			}
			else { }
			
		int oriY = location.getY();
		location.move(ranX,ranY); //move to a new location
		pirFlip(oriY);				
		foodPoints -= Tank.PIRANHA_MOVE_COST;
		color = pirColor(); //check if need to change color
	}
}