/******************************* FILE HEADER **********************************
      Main Class File:  CDDatabase.java
      File:             CD.java
      
      Author:           Data Fok / data@cs.wisc.edu
      Collaborators:    N/A

      Completion Date:  2000/12/8

      Course:           CS 302, Fall 2000, Section 14
      TA:               Jeff Shergali

      Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
      Platform:         Windows NT 4.0
**************************** 80 columns wide *********************************/

/*****************
 Represents a CD object with its data members; It contains accessor and
 mutator methods for the data members of a CD object (each containing a
 Song array)
  
 Bugs: None  			
******************/
class CD {
	
	private String title, artist; //song title and artist name
	//rawSec is the actual "total" seconds of the songs,
	//releaseYr is the release year of the CD
	private int numOfSong, totalMin=0 , totalSec=0, rawSec = 0, releaseYr;
	private SongArray songArray; 
		
	/*****************
	 Constructor - initializes all the data members to its valid states, 
	 creates a song array with an initial capacity of n, pass this 
	 array to create a MyArray object.
	 
	 @param artist the name of the aritist
	 @param yr the release year of the CD
	 @param num the number of songs initially
	******************/
	public CD(String artist, String title, int yr, int num) {
		this.title = title;
		this.artist = artist;
		numOfSong = num;
		releaseYr = yr;
		songArray = new SongArray(numOfSong);
	}
	
	/*****************
	 Constructor - initializes all the data members to its valid states, 
	 creates a song array with an initial capacity of n, pass this 
	 array to create a MyArray object.(for test use)
	******************/
	public CD() {
		this("","",0,0);
		songArray = new SongArray();		
	}
	
	/*****************
	 Calculate the total time (both in minute and second)
	******************/
	public void calTotalTime() {
		for (int i = 0; i< songArray.getSize(); i++) {
			rawSec += songArray.getSongAt(i).getTotalSec();
			totalMin = (int) Math.floor(rawSec/CDDatabase.SEC_IN_MIN);
			totalSec = (int) Math.floor(rawSec%CDDatabase.SEC_IN_MIN);
		}
	}
	
	/*****************
	 Sets the title with the passed value
	 
	 @param title the album title
	******************/	
	public void setTitle(String title) {
		this.title = title;
	}
	
	/*****************
	 Sets the artist with the passed value
	
	 @param artist the artist name
	******************/
	public void setArtist(String artist) {
		this.artist = artist;
	}
	
	/*****************
	 Sets the number of songs to the passed value
	
	 @param num the number of songs
	******************/
	public void setNumOfSong(int num) {
		numOfSong = num;
	}
	
	/*****************
	 Sets the releaseYr with the passed value
	 
	 @param yr the release year
	******************/
	public void setReleaseYr(int yr) {
	    releaseYr = yr;
	}    
	
	/*****************
	 Sets the "total" second with the passed value (test use)
	 
	 @param rSec the raw total second
	******************/
	public void setRawSec(int rSec) {
		rawSec = rSec;
	}
	
	/*****************
	 Returns Album title
	 
	 @return title
	******************/
	public String getTitle() {
		return title;
	}
	
	/*****************
	 Returns artist name
	 
	 @return artist
	******************/
	public String getArtist() {
		return artist;
	}
	
	/*****************
	 Returns the song array of the CD
	
	 @return songArray, the list of songs
	******************/
	public SongArray getSongArray() {
		return songArray;
	}
	
	/*****************
	 Returns release year
	 
	 @return releaseYr
	******************/
	public int getYear() {
		return releaseYr;
	}
	
	/*****************
	 Returns Total time in string format i.e (mm:ss)
	 
	 @return String, the string of the total time
	******************/
	public String getTotalTime() {
		String sTime, zero;
		if ( totalSec < 10) {
			zero = "0";
		}
		else {
			zero = "";
		}
		sTime = totalMin + ":" + zero + totalSec;
		return sTime;
	}

	/*****************
	 Returns number of songs
	 
	 @return numOfSong
	******************/	
	public int getNumOfSong() {
		return numOfSong;
	}
	
	/*****************
	 Returns a String for outputing the CD info to a text file
	 
	 @return String
	******************/
	public String toString() {
		String tempStr = artist + "|" + title + "|" + releaseYr + "|" + numOfSong 
		+ songArray.toString();
		return tempStr;
	}
	

}
	