/******************************* FILE HEADER **********************************
      Main Class File:  CDDatabase.java
      File:             CDArray.java
      
      Author:           Data Fok / data@cs.wisc.edu
      Collaborators:    N/A

      Completion Date:  2000/12/8

      Course:           CS 302, Fall 2000, Section 14
      TA:               Jeff Shergali

      Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
      Platform:         Windows NT 4.0
**************************** 80 columns wide *********************************/

/*****************
 Represents an array of CD; it contains methods that can manipulate 
 the array (add, remove, get and set CDs)
  
 Bugs: None  			
******************/
class CDArray {
	
	final private int DEFAULT_SIZE = 11; //defaultSize of the CD in cdArray
	
	private int numOfCD = 0; //number of CD
	
	private CD[] CDList; //declare an array of CD
	
	/*****************
	 Constructor - initializes capacity with the passed value
	 
	 @param num the initial capacity of the array 
	******************/
	public CDArray(int num) {
		CDList = new CD[num];
	}
	
	/*****************
	Sets the passed CD at the passed index, throws exception if index
	is out of bound 
	******************/	
	private void setCDAt(CD myCD, int index) {
		if ( index >= 0 && index < CDList.length) {
			CDList[index] = myCD;
		}
		else {
			System.out.println("/n The index is out of the range of " + 
			"the CDList");
			throw new ArrayIndexOutOfBoundsException();
		}
	}
	
	/*****************
	 Doubles the capacity of the array
	******************/
	private void doubleSpace() {
		CD[] tempList = new CD[CDList.length*2];
		for (int i = 0; i < CDList.length ; i++) {
			tempList[i] = CDList[i];
		}
		CDList = tempList;
	}
	
	/*****************
	 Adds the passed CD at the end; if the number of CD in the array excess
	 the capacity of th array, do doubleSpace to expand the array
	******************/	
	public void addCD(CD myCD) {
		numOfCD ++;
		if ( numOfCD > getCapacity() ) {
			doubleSpace();
		}
		setCDAt(myCD, (numOfCD-1));
				
	}
	
	/*****************
	 Adds the passed CD at the specific index of the array; if the number of
	 CD in the array excess the size of th array or the index is greater than
	 the capacity of the array, do doubleSpace to expand the array
	 (test use)
	******************/	
	public void addCD(CD myCD, int index) {
		numOfCD ++;
		
		if ( numOfCD >= getCapacity() || index >= getCapacity() ) {
			doubleSpace();
		}
		setCDAt(myCD, index);
	}
	
	/*****************
	 Removes the CD at the passed index; it does this by putting the last
	 to the index place to overwrite the unwanted CD and sets the last 
	 CD to null
	******************/	
	public void removeCDAt(int index) { 
		setCDAt(CDList[numOfCD-1], index);
		CDList[numOfCD-1] = null;
		numOfCD--;
	}
	
	/*****************
	 Returns the capacity of the array
	 
	 @return capacity the capacity of the array
	******************/
	public int getCapacity() {
		return CDList.length;
	}
	
	/*****************
	 Returns the current size of the array
	 
	 @return size the current size of the array 
	              (ie number of CD in the array)
	******************/
	public int getSize() {
		return numOfCD;
	}
		
	/*****************
	 Returns the CD at the passed index
	 
	 @return CD the CD in the passed index
	******************/
	public CD getCDAt(int index) {
		return CDList[index];
	}
	
	/*****************
	 Clear the whole array, by setting all element to null (test use)
	******************/
	public void clear() {
		for ( int i = 0 ; i < CDList.length ; i++) {
			CDList[i] = null;
		}
	}
	
	/*****************
	 Returns whether the array is an empty array
	 
	 @return boolean the "empty or not" status of the array
	******************/
	public boolean isEmpty() {
		for (int i=0; i<CDList.length; i++) {
			if ( CDList[i] != null ) {
				return false;
			}
		}
		return true;
	}
	
	/*****************
	 Returns a string for output to a text file
	 
	 return tempStr, a string that "represents" the CDs in the array
	******************/
	public String toString() {
		String tempStr = "";
		for (int i=0; i < numOfCD ; i++) {
			tempStr = tempStr + CDList[i].toString();
		}
		return tempStr;
	}
}