/******************************* MAIN HEADER **********************************
    Title:              Compact Disc Database
    Files:              CDDatabase.java, CD.java, CDArray.java, Song.java,
    					SongArray.java
    Author:             Data Fok, data@cs.wisc.edu
    Collaborators:      N/A
    
    Due Date:           11:59 PM, Friday, Dec. 8, 2000
    Completion Date:    2000/12/8
    
    Course:             CS 302, Fall 2000, Section 14	
    TA:                 Jeff Shergali
    
    Compiler:           CodeWarrior IDE 4.0 (JDK 1.2)
    Platform:           Windows NT 4.0
 **************************** 80 columns wide *********************************/

import java.io.*;
import java.util.StringTokenizer;

/*****************
  This class carries out the functions of a CDDatabase.  It allows user
  to add, remove, modify, and search CD's in the current database. It also
  supports load and save functions. 
  
  Bugs: This program assumes that the user will NOT enter the character "|"
   	    in any of the String fields, otherwise the load and save will not 
   	    work properly.
   	    
   	    For modify song number, when user enter a new number of songs that
   	    is greater than the original one, the user will be prompt to enter
   	    song details for each additional song, but will NOT ask where the
   	    song will be inserted. Rather, it will place it at the end.
   	    
  NOTES:   	    
   	    This program does NOT check for empty spaces for strings. In other
   	    words, user has to keep track of how many spaces they entered for
   	    String fields if they want to search for it.  
   	    
   	    The search mechanism does NOT ignore cases. 
******************/
class CDDatabase {

	private static BufferedReader stdin; //construct a BufferedReader for
										 //standard input
	
	public static final int DEFAULT_SIZE = 11; //initial size of database
	
	public static final double SEC_IN_MIN = 60.0; // 60 sec in a min
	
	//a flag "changed" is use to see if the database have been changed, so as
	//to prompt the user to save it when exit.
	private static boolean changed;
		
	private static CDArray cdArray; 
	
	/*****************
	 Initializes all the data members and controls the overall action
	 of the program.  It displays the main menu and calls the appropriate
	 method based on user's choice
	******************/	
	public static void main(String[] args) throws IOException {
	
		stdin = new BufferedReader( new InputStreamReader 
			  (System.in));

		changed = false; //no change made
		
		//create CD array with DEFAULT_SIZE, i.e. 11
		cdArray = new CDArray(DEFAULT_SIZE);		
		
		//the display of the main menu
		System.out.println("Welcome to the CD Database");
		
		System.out.print("\n\n");
			
		while (true) { //keeplooping until exit is selected
			
			System.out.println("###########################################");
			System.out.println("1. Add CD");
			System.out.println("2. Remove CD");
			System.out.println("3. Modify CD");
			System.out.println("4. Lookup CD");
			System.out.println("5. Display CD database");
			System.out.println("6. Load existing database from file");
			System.out.println("7. Store database to file");
			System.out.println("8. Exit");
			System.out.println("###########################################");
						
			//get user's choice
			int selection = readInt (stdin, 1, 8, "Enter your choice (1-8): ",
			  "your choice");
									
			System.out.print("\n\n");
		
			switch (selection) {
		
				case 1: add();
						break;
					
				case 2: remove();
						break;
						
				case 3: modify();
						break;
								
				case 4: lookup();
						break;
				
				case 5: displayDataBase();
						break;
					
				case 6: load();
						break;
					
				case 7: save();
						break;
			
				case 8: exit();
				
			}
	
		}
	}
		
	/*****************
	 Prompts the user to enter an integer value (with the passed String
	 as prompt title).  First it catches NumberFormatException if the 
	 number is not an integer.  If it is an integer, then it will check
	 if the input is in the range (inclusive) of the the passed values
	 (min and max). If the input is correct, then return that integer
	
	 @param BufferedReader stdin, a BufferedReader object for standard input
	 @param int min, the minimum that the input integer can be
	 @param int max, the maximum that the input integer can be
	 @param String prompt, the prompt title
	 @param String type, the "data type" of the input, display when error
	                     occurs 
	 @return num, the input number after checking for possible errors
	******************/	
	private static int readInt(BufferedReader stdin, int min, int max,
		String prompt, String type)	throws IOException {
		
		int num = 0; //stores the input number
		boolean valid = false; 
		
		do { //this loop continues until user enters a "proper" integer
			try {
				System.out.print(prompt);
				num = Integer.parseInt(stdin.readLine());
				valid = true;
			}
			
			catch (NumberFormatException e) {
				//make sure the number is in the right format
				System.out.println("Error, try again");
			}
			
			finally {
				//make sure the number is in the range
				if (num < 0) {
					System.out.println("ERROR " + type + 
					  " may not be negative");
				}
				else if (num < min || num > max) {
						System.out.println("ERROR " + type + 
						  " must be between " + min + " - " + max);
				}	 
			}
			
		} while (!valid || num < min || num > max || num < 0);
		
		return num;
	}	
	
	/*****************
	Prompts the user to enter a String (with the passed String as prompt 
	title). It makes sure that the user does not input an empty string, 
	which will screw up the save and load functions.  After checking,
	It returns the String
	        ***NOTE: It ASSUMES that the user will NOT enter a "|"***
	
	@param BufferedReader stdin, BufferedReader object for standard input
	@param prompt, the prompt title
	@return s, the input String
	******************/
	private static String readString(BufferedReader stdin, String prompt) 
		throws IOException {
		
		String s = "";
		boolean valid = true;
		
		do {//loop will continue if empty string is entered
			System.out.print(prompt);
			s = stdin.readLine();
			
			if (s.equals("")) {
				System.out.println("Can't process empty input");
				valid = false;
			}	
			else
				valid = true;
		} while (!valid);
		
		return s;
	}
		
	/*****************
	 Sets the CD (and songs) info, print the info out, and confirm the
	 user to add or not
	******************/	
	private static void add() throws IOException {
		//create a temporary CD that stores the CD in the current index
		CD cd = setCDInfo();
		
		System.out.println("New CD details:");
		
		//print the newly added CD
		printCDInfo(cd);
		
		//print its songs info
		printSongsInfo(cd);
		
		//make sure that the user only enter y or n
		boolean flag = false;
		do{
			System.out.print("Is this information correct (Y/N) ? ");
			String yesNo = stdin.readLine();
			
			//if yes, changed is set to true (for save before exit)
			if (yesNo.equalsIgnoreCase("y")) {
				System.out.println("Adding new CD to database");
				cdArray.addCD(cd); //cd is added to the cdArray
				flag = false;
				changed = true;
			}
			//if no, do not add the CD to the cdArray 	
			else if (yesNo.equalsIgnoreCase("n")) {
					System.out.println("Discarding this CD");
					flag = false;
				 }	
				 else { 
					System.out.println("Error: Please enter y or n");
					flag = true;
			     }	
		}while (flag);
		
		System.out.print("\n\n");
	}
	
	/*****************
	 Part of the add method.  Prompts users for CD info and sets them 
	 into the passed CD object. It calls the setSongInfo method to set the 
	 songs info in that CD 
	 
	 @return c, the modified CD
	******************/
	private static CD setCDInfo () throws IOException {
		
		String artist, title;
		int year, num;
		
		System.out.println("Enter details for new CD: ");		
		
		//read the CD info from user		
		artist = readString(stdin, "Enter artist name: ");
		title = readString(stdin, "Enter CD title: ");
		year = readInt(stdin, 1000, 9999, "Enter the year of " +
		"recording: ", "year of recording" );
		num = readInt(stdin, 0, Integer.MAX_VALUE,
			"Enter number of songs: ", "number of songs");
		
		System.out.print("\n\n");
		
		//create a CD with the data
		CD c = new CD(artist, title, year, num);
				
		//set the song data members by callling setSongInfo
		setSongInfo (c, 0);//start Index is 0 before any song is added
	    
	    //updates the total time
	    c.calTotalTime();
	    
	    return c;			 
	}
	
	/*****************
	 Takes in CD object and add the songs to the CD after setting the each
	 song's data members. It takes in a startIndex (which indicates where 
	 adding should begin). The adding ends when the index reaches the 
	 number of songs in the CD. 
	 
	 @param cd, the CD that the songs will be added to
	 @param startIndex, the index in the songArray that the adding begins
	******************/
	private static void setSongInfo (CD cd, int startIndex)
		 throws IOException {
		
		//creates a temporary songArray for that CD
		SongArray songArray = cd.getSongArray();
		
		//prompt user to input Minutes and Seconds for each song
		//(to be added or modified)
		for (int i = startIndex; i < cd.getNumOfSong(); i++) {
			
			String name="";
			int min = 0, sec = 0;
			
			//gets the song title			
			System.out.println("Enter song details for song #" + (i+1));
			name = readString(stdin, "Enter the title of song: ");
			
			//asks user to input times for the new song
			System.out.println("Enter playing time details for new song");
			min = readInt(stdin, 0, Integer.MAX_VALUE, "Enter the " +
				  "minutes: ", "minutes");
			sec = readInt(stdin, 0, 59, "Enter the seconds: ",
				  "seconds");
						
			//creates a temporary Song that stores the Song in each loop
			Song song = new Song(name, min, sec);
			
			songArray.addSong(song);
			
			System.out.print("\n\n");
	    }
	}
	
	/*****************
	 Prints the passed CD info
	  	 
	 @param cd, a CD to be printed 
	******************/
	private static void printCDInfo(CD cd) {
		System.out.println("Artist: " + cd.getArtist() );
		System.out.println("CD title: " + cd.getTitle() );
		System.out.println("Year: " + cd.getYear() );
		System.out.println("Total Time: " + cd.getTotalTime() );
		System.out.println("Number of Songs: " + cd.getNumOfSong() );
	}
	
	/*****************
	 Prints the passed CD's Songs info
	 
	 @param cd, a CD that contains the songs to be printed
	******************/
	private static void printSongsInfo(CD cd) {
		//create a temporary songArray of that passed CD
		SongArray tempSongs = cd.getSongArray();
		
		//won't print if no song in CD
		if (!tempSongs.isEmpty()) {
			//print each song's info
			for (int i = 0; i < tempSongs.getSize() ; i++) {
				Song song = tempSongs.getSongAt(i);
				System.out.println("Song #" + (i+1) + ": " + song.getName() +
				  ", " + song.getTime() );
			}
		}
		else { }
	}
	
	/*****************
	 Removes the CD choosed by user. User are prompt to search which CD to
	 remove by CD title. It then create an int array to store the
	 found index in the cdArray.It then display all cd match by the search 
	 method. User are prompt to select which song to be removed (if more than 
	 one match). CD is then removed from the cdArray.	 
	******************/
	private static void remove() throws IOException{
		
		if ( !cdArray.isEmpty() ) {
		
			int numMatches=0, interest;
		
			//create array of int to store the found index in cdArray
			//with capacity in case all of the cases are matched
			int[] array = new int[cdArray.getCapacity()];
		
			//set all values to -1, for checking use
			for (int i = 0 ; i < array.length ; i++){
				array[i]= -1;
			}
			
			System.out.println("Searching by CD Title");
			System.out.print("\n\n");
			numMatches = lookupByCDTitle(array);
			if (numMatches == 0) {
				printNoMatches();
			}
			else {
				printCDTitleResults(numMatches, array);
								
				if (numMatches ==1) { //if only one CD found just remove it
									  //without prompt the user to chose
					interest = 1;
				}
				else {
					//get which CD to remove
					interest = readInt(stdin, 1, numMatches ,
				  	  "Choose the CD # of interest: ", "CD #");
				  	System.out.print("\n");  
				}
				
				System.out.println("Removed CD");
				printCDInfo(cdArray.getCDAt(array[interest-1]));
				cdArray.removeCDAt(array[interest-1]);
			}
		
			changed = true;
		}
		else {
			System.out.println("Cannot remove CDs from an empty database");
		}
		System.out.print("\n\n");
	}
	
	
	
	/*****************
	 Display the information of all CD in the CDDatabase. As long as there are
	 CD, it will print out all the info of the CD (by printCDInfo method), 
	 and its song info (by printSongInfo method)
	******************/	
	private static void displayDataBase() {
		int cdIndex = 1;
		
		//won't do if no CD in database
		if ( !cdArray.isEmpty() ) {
			for( int i=0 ; i < cdArray.getSize() ; i++) {
				CD tempCD = cdArray.getCDAt(i);
				
				System.out.println("CD #" + cdIndex);
				printCDInfo(tempCD);
				printSongsInfo(tempCD);
				
				cdIndex += 1;
				System.out.print("\n\n");
			}
		}
		else {
			System.out.println("No CDs in collection");
			System.out.print("\n\n");
		}
	}
	
	/*****************
	 Lookup the CD to be modified by CD title. If match is found, print 
	 out the CD's info  and display the Modify Menu. It then calls the 
	 appropriate methods according to user's choice
	******************/					
	private static void modify() throws IOException {
		
		//won't do if empty cdArray
		if ( !cdArray.isEmpty() ) {
		
			int numMatches=0, modifyIndex;
		
			//create array of int to store the found index in cdArray
			//with capacity in case all of the cases are matched
			int[] array1 = new int[cdArray.getCapacity()];
		
			//set all values to -1, for checking use
			for (int i = 0 ; i < array1.length ; i++){
				array1[i]= -1;
			}
		
			System.out.println("Searching by CD Title");
			System.out.print("\n\n");
			numMatches = lookupByCDTitle(array1);
			if (numMatches == 0) {
				printNoMatches();
			}
			else{
				printCDTitleResults(numMatches, array1);
				
				//get which CD to modify
				modifyIndex = readInt(stdin, 1, numMatches, 
				  "Choose the CD # of interest: ", "CD #");
				System.out.print("\n");
		 
				//Get the modify CD
				CD modifyCD = (cdArray.getCDAt(modifyIndex-1));	 
				//Print that CD's info
				printCDInfo (modifyCD);
				printSongsInfo (modifyCD);
				System.out.println("");
			
				//Go back to the Modify Menu once modifying is done (unless 
				//option 4 is chosen, which will go back to the Main Menu)
				boolean repeat = true;
		
				do{	
				
					System.out.println ("Enter item to modify");
					System.out.println ("1. Change Basic Information");
					System.out.println ("2. Change Number of Songs");
					System.out.println ("3. Change a Song");
					System.out.println ("4. Return to Main Menu\n");
				
					int choice = readInt (stdin, 1, 4, "Enter your choice: ", 
					  "your choice");
					System.out.println("");
					switch (choice) {
					
						//all cases will change the variable "changed" to true
						//except case 4
						case 1: 
							modifyBasic(modifyCD);
							changed = true;
							break;
									
						case 2:
							printSongsInfo (modifyCD);
							System.out.println("");
							//get the new number of songs from the user 
							//(assume max songs is what integer can hold)
							int newNumSong = readInt (stdin, 1, 
							  Integer.MAX_VALUE, "Enter new number of songs: "
							  , "new song number\n\n");
							modifySongNumber(newNumSong, modifyCD);
							changed = true;							
							break;
		
						case 3: 
							printSongsInfo(modifyCD);
							System.out.println("");
							modifySong(modifyCD);
							changed = true;
							break;
					
						case 4: 
							repeat = false;
							break;
					
					}			
						
				} while (repeat); 
			}
		
		}
		else {
			System.out.println("Cannot modify CDs from an empty database\n");
		}
					
	}
	
	/*****************
	 The Modify Basic information option of the modify method. Sets 
	 the artist name, CD title and  release year to
	 the passed CD's variable, and prints the updated info
	 
	 @param cd, a CD to be modified
	******************/	
	private static void modifyBasic (CD cd) throws IOException {
		cd.setArtist(readString (stdin, "Please enter new Artist Name: "));
		cd.setTitle(readString (stdin, "Please enter new CD Title: "));
		cd.setReleaseYr(readInt (stdin, 1000, 9999,
			 "Please enter new year: ", "new year"));
		
		System.out.println("");
		printCDInfo(cd);
		System.out.println("");
	}
	
	/*****************
	 The Modify Song Number option of the modify method. It takes in a song 
	 number and compares it to the number of songs in the passed CD. It adds,
	 removes, or doesn't change songs to the passed CD depending on how
	 the numbers compare
	 
	 @param newNumSong, the new number of songs
	 @param modifycd, the CD to be modified	 
	******************/	
	private static void modifySongNumber (int newNumSong, CD modifycd)
	    throws IOException {
		
		if (newNumSong > modifycd.getNumOfSong()) {
			addSongs(newNumSong, modifycd);
		}
		else if (newNumSong < modifycd.getNumOfSong()) {
				removeSongs(newNumSong, modifycd);
			 }
			 //newNumSong == original number of song
			 else {
				System.out.println("There is no change made.\n");
			 }
		
		//print the updated info
		printCDInfo (modifycd);
		printSongsInfo (modifycd);
		System.out.println("");
					
	}

	/*****************
	 Part of the modifySongNumber method. It sets the passed new number 
	 of songs to the passed CD and set each additional song's info by 
	 calling the setSongInfo method.
	 
	 @param newNumSong, the new number of songs
	 @param modifycd, the CD to be modified	 
	******************/
	private static void addSongs (int newNumSong, CD modifycd)
		throws IOException {	
		
		//set the new song number
		modifycd.setNumOfSong(newNumSong);
		
		//expand the songArray depends on how many song will be added
		modifycd.getSongArray().addNumSong(newNumSong - 
		  modifycd.getSongArray().getSize());
						
		//set each song's info; pass the CD, and the
		//"end index" to setSongInfo method
		setSongInfo (modifycd, modifycd.getSongArray().getSize());
					
		System.out.println("");
		
		//calculate total time
		modifycd.calTotalTime();
	}

	/*****************
	 Part of the modifySongNumber method. It sets the passed new number 
	 of songs to the passed CD and removes the extra ones based on 
	 User's choice. An updated list is printed out each time the user 
	 removes a song.
	 
	 @param newNumSong, the new number of songs
	 @param modifycd, the CD to be modified
	******************/	
	private static void removeSongs(int newNumSong, CD modifycd)
		throws IOException {
		
		
		
		//create a temporary array to store the songarray
		SongArray tempSongArray = modifycd.getSongArray();
		
		modifycd.setNumOfSong(newNumSong);
		
		//this variable keep track of number of songs left after each remove
		int tempSongNumber = tempSongArray.getSize();
		
		//remove extra songs until the number of songs left matches the new
		//number of songs
		for (int i = (tempSongArray.getSize()); i > (newNumSong); i--) {
			
			printSongsInfo(modifycd);
			
			//prompt the user to input which song to remove
			int removeIndex = (readInt(stdin, 1, tempSongNumber,
			 "Enter song number to remove: ", "song number")-1); //actual index is 1 smaller
												  				 //than the user's input
			//print the deleted song
			System.out.println("Removed Song #" + (removeIndex+1) + " " +
				 (tempSongArray.getSongAt(removeIndex).getName() ) + ", " +
				 (tempSongArray.getSongAt(removeIndex).getTime() ) + "\n");
			
			tempSongNumber--;
			
			//remove the song from the array
			tempSongArray.removeSongAt(removeIndex);
						
			//print the updated list of songs
			printSongsInfo (modifycd);
			
			//calculate total time
			modifycd.getTotalTime();			
			System.out.println("");									
		}
		//reduce the size of the songArray to how many songs a CD have
		modifycd.getSongArray().roundSize();
		
	}
	
	/*****************
	 The Modify Song option of the modify method. It prompts the user for 
	 which song to modify and then prompts to enter the new info for that
	 song. It then prints the updated info after setting them
	 
	 @param modify, the CD to be modified
	******************/									
	private static void modifySong (CD modifycd) throws IOException {
		
		//the song to be changed
		int songNumber = readInt (stdin, 1, 
			modifycd.getSongArray().getSize(),
			"Enter the song number to change: ", "song number");
		
		//create a temp song with the correct index (number-1)
		Song tempSong = 
			(modifycd.getSongArray().getSongAt(songNumber-1));
			
		//prompt user to enter the new details and sets them
		System.out.println("\nEnter song details for song #" + (songNumber));
		tempSong.setName(readString (stdin, "Song name: "));
		System.out.println("Enter playing time details for new song");
		//assume the min of a song is what an Integer can hold
		int min = readInt(stdin, 0, Integer.MAX_VALUE, "Enter the " +
				  "minutes: ", "minutes");
		int sec = readInt(stdin, 0, 59, "Enter the seconds: ",
				  "seconds");
		tempSong.setTime(min, sec);
				
		modifycd.calTotalTime();
		System.out.println("");
		
		//print the updated CD and songs info
		printCDInfo (modifycd);	
		printSongsInfo (modifycd);
		System.out.println("");		
	
	}		

	
	
	/*****************
	 Displays the LookUp menu and prompts user for a choice. Calls the 
	 appropriate search mechanism by passing an array of int values
	 (all values are set to -1). When done, returns this array.
	 
	 @return array, a int array that contains the index of matches
	******************/
	private static int[] lookup() throws IOException {
	
		//won't do if database is empty
		if ( !cdArray.isEmpty() ) {
		
			boolean menuloop = false;
			int choice=0;
			
			do {
				try {	
					//print the lookup meny
					System.out.println ("Enter search mechanism");
					System.out.println ("1. Lookup by Artist");
					System.out.println ("2. Lookup by CD Title");
					System.out.println ("3. Lookup by Song Name");
					System.out.println ("4. Return to the Main Menu");
				    System.out.print("\n");
				    System.out.print("Enter your choice (1 - 4): ");
				    choice = Integer.parseInt(stdin.readLine());
				    menuloop = false;
				}
				
				catch (NumberFormatException e) {
					//make sure the number is in the right format
					System.out.println("Error, try again");
				}

				finally {
				//make sure the number is in the range
					if (choice < 0) {
						System.out.println("ERROR your choice" + 
						  " may not be negative");
						menuloop = true;  
						System.out.println("\n");
					}
					else if (choice < 1 || choice > 4 ) {
							System.out.println("ERROR your choice" + 
							 " must be between " + 1 + " - " + 4);
							menuloop = true;  
							System.out.println("\n");
					}	 
				}	
			} while(menuloop);
		
			//create array of int to store the found index in cdArray
			//with capacity in case all of the cases are matched
			int[] array = new int[cdArray.getCapacity()];
		
			//set all values to -1, for checking use
			for (int i = 0 ; i < array.length ; i++){
				array[i]= -1;
			}
			
			int numMatches = 0;//stores the number of matches
		
			switch (choice) {
				
				case 1: numMatches = lookupByArtist(array);
						if (numMatches == 0) {
							printNoMatches();
						}
						else {
							printArtistResults(numMatches, array);
						}		
						break;
								
				case 2: numMatches = lookupByCDTitle(array);
						if (numMatches == 0) {
							printNoMatches();
						}
						else {
							printCDTitleResults(numMatches, array);
						}		
						break;	
					
				case 3: //get a song name from the user(pass to 
						//lookupBySongName)
						String songName = readString
						  (stdin, "Enter the song name: ");
			
						numMatches = lookupBySongName(array, songName);
					
						if (numMatches == 0) {
							printNoMatches();
						}
						else {
							printSongNameResults(numMatches, array, songName);
						}
						break;			
				case 4: break;
			}
			
			return array;
		
		}
		else {
			System.out.println("Cannot do lookup on empty database");
			System.out.print("\n\n");
			return null;
		}		
		
	}
	
	/*****************
	 Prompts the user for an artist name and search the CDDatabase for 
	 matches. It takes in a int array (all -1 initially) and sets
	 the indexs of the found CD to the array. It returns the number of
	 matches found (the indexs of the found CD in the int array)
	 
	 @param b, an int array that indicates the matching indexs at the
	 		   end of the method.
 	 @return numMatches, the number of matches	 		   
	******************/	
	private static int lookupByArtist(int[] b) throws IOException {
		int numMatches = 0; //stores the number of matches
		
		//prompt user for artist name
		String artistName = readString(stdin, "Enter artist name: ");
		
		//search the current database; for each match found, set the int
		//array's value to that index, increment numMatches
		for (int i = 0; i < cdArray.getSize(); i++) {
			if((cdArray.getCDAt(i)).getArtist().equals(artistName)) {
				b[numMatches] = i; 
				numMatches++;
			}									
		}
		return numMatches;
	}
	
	/*****************
	 **Must be excuted after the lookupByArtist method**
	 Prints the search results from the lookupByArtist method in the format
	 specified in the requiements. It prints all the CD's that are greater 
	 than -1 in the int array (that is the index of the found CD).
	 
	 @param numMatches, the number of matches found in the database
	 @param b, the int array that contains the indexs of the found CD
	******************/	
	private static void printArtistResults(int numMatches, int[] b) {
		int matches = 1; //Match number in the header of the printed results
		
		System.out.println("Number of matches is " + numMatches + "\n\n");
		for (int i = 0; i < b.length; i++) {
			//only prints those that are indicated by value greater than -1
			if (b[i] > -1) {
				System.out.println("#### Match " + matches + " ####");
				printCDInfo(cdArray.getCDAt(b[i]));
				System.out.print("\n\n");							
				matches++;
			}
		}		
	}
	
	/*****************
	 Prompts the user for a CD title and search the CDDatabase for 
	 matches. It takes in a int array (all -1 initially) and sets
	 the indexs of the found CD to the array. It returns the number of
	 matches found (the indexs of the found CD in the int array)
	 
	 @param b, an int array that indicates the matching indexs at the
	 		   end of the method.
 	 @return numMatches, the number of matches	 
	******************/
	private static int lookupByCDTitle (int[] b) throws IOException {
		int numMatches = 0; //stores the number of matches
		
		//prompt user for CD Title
		String CDTitle = readString(stdin, "Enter CD Title: ");

		//search the current database; for each match found, set the boolean
		//array's value to true at that index, increment numMatches
		for (int i = 0; i < cdArray.getSize(); i++) {
			if((cdArray.getCDAt(i)).getTitle().equals(CDTitle)) {
				b[numMatches] = i;
				numMatches++;
			}									
		}
		return numMatches;
	}
	
	/*****************
	 **Must be excuted after the lookupByCDTitle method**
	 Prints the search results from the lookupByCDTitle method in the format
	 specified in the requiements. It prints all the CD's that are greater 
	 than -1 in the int array (that is the index of the found CD).
	 
	 @param numMatches, the number of matches found in the database
	 @param b, the int array that contains the indexs of the found CD	 
	******************/
	private static void printCDTitleResults (int numMatches, int[] b) {
		int matches = 1; //Match number inthe header of the printed results
		
		System.out.println("Number of matches is " + numMatches + "\n\n");
		for (int i = 0; i < b.length; i++) {
			//only prints those that are indicated by value greater than -1
			if (b[i] > -1) {
			
				System.out.println("#### Match " + matches + " ####");
				
				//create a temporary CD
				CD tempCD = (cdArray.getCDAt(b[i]));
				
				//print the matching CD
				printCDInfo(tempCD);
		
				//print its songs info
				printSongsInfo(tempCD);						
				
				System.out.print("\n\n");
				
				matches++;		
			}
		}
	}
	
	/*****************
	 Takes in a song name and search the CDDatabase for 
	 matches. It takes in a int array (all -1 initially) and sets
	 the indexs of the found CD to the array. It returns the number of
	 matches found (the indexs of the found CD in the int array)
	 
	 @param b, an int array that indicates the matching indexs at the
	 		   end of the method.
	 @param songName, the song name to be searched		   
 	 @return numMatches, the number of matches	 
	******************/	
	private static int lookupBySongName(int[] b, String songName)
		throws IOException {
		
		int numMatches = 0; //stores the number of matches
 		 		
		for (int i = 0; i < cdArray.getSize(); i++) {
			//create a temporary CD
			CD tempCD = cdArray.getCDAt(i);
			
			//search each song
			for (int j = 0; j < tempCD.getSongArray().getSize(); j++) {
				
				//for each match found, set the int array's value to
				//that index, increment numMatches				
				if((tempCD.getSongArray().getSongAt(j)).getName()
				  .equals(songName)) {
					b[numMatches] = i;
					numMatches ++;
		
				}
			}
		}
		return numMatches;
	}
	
	/*****************
	 **Must be excuted after the lookupBySongName method**
	 Prints the search results from the lookupBySongName method in the format
	 specified in the requiements. After locating the CD with the song (>-1
	 in the passed array), it locate the exact position of the passed 
	 songName in that CD.  It prints all the CD's that are indicated
	 by a value that > -1 in the passed int array with that song's info.
	 	 
	 @param songName, the song name to be searched
	 @param b, the int array that contains the "indications"	 
	******************/
	private static void printSongNameResults (int numMatches, 
	  int[] b, String songName) {
		
		int matches = 1; //Match number in the header of the printed results
		
		System.out.println("Number of matches is " + numMatches + "\n\n");
		for (int i = 0; i < b.length; i++) {
			
			//only prints those that are indicated by value greater than -1
			if (b[i] > -1) {
				
				//create a temporary CD
				CD tempCD = (cdArray.getCDAt(b[i]));
				
				System.out.println("#### Match " + matches + " ####");
				printCDInfo(tempCD);
				
				//locate that song in this CD
				SongArray tempSongs = tempCD.getSongArray();
				
				for (int j = 0; j < tempSongs.getSize(); j++) {
					
					if (tempSongs.getSongAt(j).getName().equals(songName)) {
						//print that song
						System.out.println("Song: " + 
					      tempSongs.getSongAt(j).getName() + ", " 
					      + tempSongs.getSongAt(j).getTime() );								
					}
					System.out.print("\n\n");							
					matches++;
				}
			}
		}
	}
	
	/*****************
	 prints the no match message
	******************/
	private static void printNoMatches() {
		System.out.println ("No matches found");
		System.out.print("\n\n");
	}
	
	
	
	/*****************
	 Loads the source file by tokenizing the string. It sets all the info
	 into a tempCD and add it to the cdArray in this database.
	 **An error will NOT occur if there are more than one "|" separating 
	   the fields
	******************/	
	private static void load() throws IOException{
		CD tempCD;
		SongArray tempSongs;
		
		
		File srcFile = new File (verifyLoadFile());//File with valid name
			
		//BufferedReader for reading information from file
		BufferedReader inFile = new BufferedReader(new FileReader(srcFile));
		
		while(inFile.ready()) { //as long as there are more lines to read
			
			String artist, title;
			int yr, num;
			
			String line = inFile.readLine();
			
			//check for empty lines
			if (line.equals("")) {
				break;
			}
			
			//create a StringTokenizer object with that line (no delimiters
			//at this time (except default)) Each "nextToken" will have its
			//own delimiters ("|" or ":")
			StringTokenizer t = new StringTokenizer (line);
			
			//set the CD data members with the next token
			artist = t.nextToken("|");
			title = t.nextToken("|");
			yr = Integer.parseInt(t.nextToken("|"));
			num = Integer.parseInt(t.nextToken("|"));
			
			//create a temp CD
			tempCD = new CD(artist, title, yr, num); 
			//create a SongArray for the song array of this CD
			tempSongs = tempCD.getSongArray();
			
			//set each song's data member in that CD
			for (int i = 0; i < tempCD.getNumOfSong(); i++) {
				String name;
				int min, sec;
				
				//set the song data with the next token
				name = t.nextToken("|");
				min = Integer.parseInt(t.nextToken(":|"));
				sec = Integer.parseInt(t.nextToken(":|"));
				
				//create a temporary Song
				Song tempSong = new Song(name, min, sec);
				
				//add the tempSong to the tempSongs (the MyArray)
				tempSongs.addSong(tempSong);
			}
			//calculate total time
			tempCD.calTotalTime();
			
			//add the tempCD to the cdArray in this database
			cdArray.addCD(tempCD);
		}
		inFile.close();
		System.out.println("Added info to database");
		System.out.print("\n\n");
		changed = true;
	}
	
	/*****************
	 Verifies the source file to be loaded is in the right format. It
	 returns the filename of the checked file
	 
	 @return filename, the filename of the checked source
	******************/
	private static String verifyLoadFile () throws IOException {
		File srcFile; //File object
		String filename; //stores the filename
				
		boolean repeat = false;
		
		//repeat until a right file name is entered.
		do{
			filename = readString (stdin,
				"Enter the source file pathname: ");
			
			//create a new file with that filename	
			srcFile = new File(filename); 
			repeat = false;
			
			if (!srcFile.exists()) {
				System.out.println("This file does not exist");
				repeat = true;
			}
			else if (!srcFile.isFile() || !srcFile.canRead() ||
				!FileFormatVerifier.verify(filename)) {
				
				System.out.println("Error, file not readable");
				repeat = true;
			}
			else 
				System.out.println("");
		
		} while(repeat);
		
		return filename;
	}	
	
	
	
	/*****************
	 Saves the data in the database to a file using a PrintWriter object.
	******************/
	private static void save() throws IOException {
		
		//check if the database is empty
		if ( !cdArray.isEmpty() ) {
			//create a file with the verified name
			File dstFile = new File (verifySaveFile());

			PrintWriter outFile = new PrintWriter (new BufferedWriter
				(new FileWriter (dstFile)));
		
			//print the data of the cdArray to outfile
			for (int i=0; i < cdArray.getSize() ; i++) {
				outFile.println(cdArray.getCDAt(i).toString() + "|");
			}
			
			outFile.close();
			System.out.println("Saved current database");
		}
		else {
			System.out.println("Cannot save an empty database");
		}	
		System.out.print("\n\n");
		changed = false;//set change back to false
	
	}
	
	/*****************
	 Verify that the destination file is writable. If the file already 
	 exists, prompt the user to overwrtie it or not.
	 
	 @return filename, the filename of the destination file in which data
	  		           will be stored
	******************/	
	private static String verifySaveFile() throws IOException {
		File dstFile; 
		String filename;  //stores the file name
		boolean overwrite = false;
		boolean repeat = false;
		//this loop continues if the user does not want to overwrite file
		do {
			
			//this loop continues until the user enter a right filename
			do {
				filename = readString (stdin,
					 "Enter the pathname of the destination file: ");			
				
				dstFile = new File (filename);
				if (dstFile.exists() && (dstFile.isDirectory() || 
					!dstFile.canWrite())) {
					
					System.out.println("Error, file not writable");
					repeat = true;
				}
				else {
					repeat = false;
				}
			} while (repeat);
		
			if (dstFile.exists()) {
				System.out.print("File already exists, overwrite? (Y/N): ");
				boolean loop = false;
				String yesNo = stdin.readLine();
				
				//this loop makes sure that the user enter y or n
				do {
					if (yesNo.equalsIgnoreCase("y")) {
						loop = false;
						overwrite = false;
					}	
					else if (yesNo.equalsIgnoreCase("n")) {
						loop = false;
						overwrite = true;
					}
					else {
						System.out.print("Error, please enter y or n: ");
							loop = true;	
					}
				} while (loop);
			}
			else { //the file doesn't exist, break out of loop
				break;
			}	
				
		} while (overwrite);
		
		return filename;
	}					
	
	/*****************
	Exits the program; If changes have been made since the last save (which 
	is kept track by the boolean value "changed"), then it will prompt the 
	user to save or not.  Otherwise quit without asking.
	******************/	
	private static void exit() throws IOException {
		
		boolean repeat = false;//check for input error
		
		if(changed) {//if there is change
			System.out.print("The database has changed since store. " +
			  "Store before exit? (Y/N): ");
			
			//this loop will continue until the user inputs y or n
			do {
				String yesNo = stdin.readLine();
				if (yesNo.equalsIgnoreCase("y")) {
					save();
					System.out.println("\nDatabase successfully shut down.");
					System.exit(0);
				}	
				else if (yesNo.equalsIgnoreCase("n")) {
					  System.out.println("\nDatabase successfully shut down.");
					  System.exit(0);
					 }
					 else { //input error
				  		System.out.println("Error, please enter y or n:");
				 		repeat = true;	
					 }
			} while (repeat);
		} 	
		else { //no change
			System.out.println("Database successfully shut down.");
			System.exit(0);
		}		
	}
	
	
	
	
}
