
/*******************************************************************************

Program:          CDLibrary

Author:           Chi-Kai (Clement Chow) chikaichow@students.wisc.edu
Collaborators:    David Schneider (TA)

Due Date:         Friday, December 17th, 5:00 pm
Completion Date:  Wednesday December 15th
				  (updated Friday, December 17th)

Course:           CS 302, Fall '99, Section 17
TA:               David Schneider
Assignment:       Project 2

Compiler:         CodeWarrior (JDK 1.2)
Platform:         Windows NT 4.0


<--------------Code should not be any wider than this line--------------------->
*******************************************************************************/

import java.io.*;
import java.lang.*;
import java.util.*;

/*****************
  This class carries out the functions of a CD Library.  It allows user
  to add, remove, modify, and search CD's in the current library. It also
  supports load and save functions. 
  
  Bugs: This program assumes that the user will NOT enter the character "|"
   	    in any of the String fields, otherwise the load and save will not 
   	    work properly.
   	    
   	    For modify song number, when user enter a new number of songs that
   	    is greater than the original one, the user will be prompt to enter
   	    song details for each additional song, but will NOT ask where the
   	    song will be inserted. Rather, it will place it at the end. (i just
   	    found out this requirement from the FAQ!!)  
   	    
  NOTES:
   	    I'm sure there is a better way to implement the search function.
   	    For mine, user will be given a "disc number" for each match. If 
   	    there are more than one match, then it will display all the matches,
   	    each with its own "disc number" (which corresponds to the index of 
   	    that CD in the current library.) I implement the code so that
   	    user will only be able to choose those that are listed. There
   	    are no errors, but I think it's not very "user-friendly."
   	    
   	    I included the field total time in the display for "search by
   	    Artist" because I think this would be one of the useful ways for 
   	    the user to identify the right CD if there are more than one 
   	    matches.
   	    
   	    Each CD can hold no more than 100 songs, which I think is a 
   	    reasonable limit. However, this can easily be changed (look at
   	    the constant varaible "MAX_NUM_SONGS")			
   	    
   	    This program does NOT check for empty spaces for strings. In other
   	    words, user has to keep track of how many spaces they entered for
   	    String fields if they want to search for it.  
   	    
   	    The search mechanism does NOT ignore cases. 
******************/
class CDLibrary {
	
	private static BufferedReader stdin; //buffered reader for standard input	
	
	private static MyArray myCDs;    //MyArray object that stores the current
								     //CD array
	public static final int DEFAULT_SIZE = 10; //initial CD and Song array
											   // capacity
	public static final int SEC_IN_MIN = 60;   //60 seconds in a minute
	
	private static final int MAX_TOTAL_TIME = (80*60); //Max toatl time for a
											//CD (80 min * 60sec/min) = 4800s
	private static final int MAX_NUM_SONGS = 100; //Max number of songs that
											//a CD can hold
	
	private static int index;  //index that keeps track of the current
							   //"size" of the CD array
	private static boolean change;  //keep track if any changes have
									//been made since the last save

	
	/*****************
	 Initializes all the data members and controls the overall action
	 of the program.  It displays the main menu and calls the appropriate
	 method based on user's choice
	******************/		
	public static void main (String args []) throws IOException {
		
		stdin = new BufferedReader (new InputStreamReader(System.in));	
		index = 0; //the Library is "empty"
		change = false;//no change has been made yet
		
		//create CD array with DEFAULT_SIZE		
		CD [] cds = new CD [DEFAULT_SIZE];
		
		//initialize each CD in the array with DEFAULT_SIZE number of songs
		for (int i = 0; i < cds.length; i++) {
			cds[i] = new CD(DEFAULT_SIZE);
		}
		
		//create MyArray object with the CD array just created
		myCDs = new MyArray (cds);
		
		while(true) {//loop until exit
			
			//show the main menu
			mainMenu();
			
			//get user's choice
			int choice = readInt (stdin, 1, 7, "Enter your choice: ");
			System.out.println("");
			
			//call appropriate methods
			switch (choice) {
		
				case 1: add();
						break;
						
				case 2: remove();
						break;
						
				case 3: modify();
						break;
					
				case 4: lookup();						
						break;
				
				case 5: load();
						break;
						
				case 6: save();
						break;				
					
				case 7: exit();
									
			}
		}
	}

	/*****************
	Exits the program; If changes have been made since the last save (which 
	is kept track by the boolean value "change"), then it will prompt the 
	user to save or not.  Otherwise quit without asking.
	******************/		
	private static void exit () throws IOException {
		
		boolean repeat = false;//check for input error
		
		if(change) {//if there is change
			System.out.print(
				"Database has been modified, save changes? [Y or N]: ");
			
			//this loop will continue until the user inputs y or n
			do {
				String yesNo = stdin.readLine();
				if (yesNo.equalsIgnoreCase("y")) {
					save();
					System.out.println("\nDatabase successfully shut down.");
					System.exit(0);
				}	
				else if (yesNo.equalsIgnoreCase("n")) {
					System.out.println("\nDatabase successfully shut down.");
					System.exit(0);
				}
				else { //input error
					System.out.println("Error, please enter y or n:");
					repeat = true;	
				}
			} while (repeat);
		} 	
						
		else{ //no change
			System.out.println("Database successfully shut down.");
			System.exit(0);
		}		
	}
	
	/*****************
	 Prompts the user to enter an integer value (with the passed String
	 as prompt title).  First it catches NumberFormatException if the 
	 number is not an integer.  If it is an integer, then it will check
	 if the input is in the range (inclusive) of the the passed values
	 (min and max). If the input is correct, then return that integer
	
	 @param BufferedReader stdin, a BufferedReader object for standard input
	 @param int min, the minimum that the input integer can be
	 @param int max, the maximum that the input integer can be
	 @param String prompt, the prompt title
	 @return num, the input number after checking for possible errors
	******************/		
	private static int readInt (BufferedReader stdin, int min, int max,
		String prompt)	throws IOException {
		
		int num = 0; //stores the input number
		boolean valid = false; 
		
		do { //this loop continues until user enters a "proper" integer
			try {
				System.out.print(prompt);
				num = Integer.parseInt(stdin.readLine());
				valid = true;
			}
			
			catch (NumberFormatException e) {
				//make sure the number is in the right format
				System.out.println("Error, try again\n");
			}
			
			finally {
				//make sure the number is in the range
				if (num < min || num > max) {
					System.out.println("Please enter a number between "
					 + min + " and " + max + "\n");
				}	 
			}
			
		} while (!valid || num < min || num > max);
		
		return num;
	}
	
	/*****************
	Prompts the user to enter a String (with the passed String as prompt 
	title). It makes sure that the user does not input an empty string, 
	which will screw up the save and load functions.  After checking,
	It returns the String
	        ***NOTE: It ASSUMES that the user will NOT enter a "|"***
	
	@param BufferedReader stdin, BufferedReader object for standard input
	@param prompt, the prompt title
	@return s, the input String
	******************/		
	private static String readString (BufferedReader stdin, String prompt) 
		throws IOException {
		
		String s = "";
		boolean valid = true;
		
		do {//loop will continue if empty string is entered
			System.out.print(prompt);
			s = stdin.readLine();
			
			if (s.equals("")) {
				System.out.println("Invalid entry, please try again.\n");
				valid = false;
			}	
			else
				valid = true;
		} while (!valid);
		
		return s;
	}
 			
	/*****************
	 Part of the add method.  Prompts users for CD info and sets them 
	 into the passed CD object. It calls the setSongInfo method to set the 
	 songs info in that CD 
	******************/
	private static void setCDInfo (CD c) throws IOException {
		System.out.println("Enter details for new CD: ");		
		
		//set the CD data members		
		c.setArtist(readString (stdin, "Enter artist name: "));
		c.setAlbumTitle(readString (stdin, "Enter CD title: "));
		c.setReleaseYr(readInt (stdin, 1000, 9999, "Enter year: "));
		c.setNumSongs(readInt (stdin, 1, MAX_NUM_SONGS,
			"Enter number of songs: "));
		//error check, 100 is a reasonable maximum number of song in the CD
		
		//set the song data members by callling setSongInfo
		setSongInfo (c, 0, 0);//total time and start Index are both 0 before
							  //any song is added
	    
	    //add this CD into the library (if later confirm "NO", then remove
	    //it at that time)
	    myCDs.addElement(c);
	    
	    //updates the total time
	    c.getTotalTime();
	    
	    //increment the current "size" of the CD array in this library
	    index++;			 
	}
	
	/*****************
	 Takes in CD object and add the songs to the CD after setting the each
	 song's data members. It takes in a startIndex (which indicates where 
	 adding should begin) and a totalTime so far in that CD. This totalTime
	 is used to make sure that the the total time of that CD never exceeds
	 80 minutes.The adding ends when the index reaches the number of songs
	 in the CD. 
	 
	 @param cd, the CD that the songs will be added to
	 @param toatlTime, the totalTime so far in that CD
	 @param startIndex, the index in the songArray that the adding begins
	******************/	
	private static void setSongInfo (CD cd, int totalTime, int startIndex)
		 throws IOException {
		
		//creates a temporary songArray for that CD
		MyArray songs = cd.getMySongArray();
		
		//variables that will store user's input time
		int tempMinute = 0;
		int tempSecond = 0;
		
		//prompt user to input Minutes and Seconds for each song
		//(to be added or modified)
		for (int i = startIndex; i < cd.getNumSongs(); i++) {
			
			//creates a temporary Song that stores the next Song each loop
			Song song = (Song) songs.getElementAt(i);
			
			//sets the song title			
			System.out.println("\nEnter song details for song " + (i+1));
			song.setSongTitle(readString (stdin, "Song name: "));
			
			//this loop asks user to input times for the new song; it
			//will re-ask the user for input if the total time is over the 
			//80-minutes limit.
			boolean repeat = false;
			do {
				System.out.println("Enter Time details for new song:");
				tempMinute = readInt (stdin, 0, 79, "Minutes: ");
				tempSecond = readInt (stdin, 0, 59, "Seconds: ");
				totalTime += (tempMinute*SEC_IN_MIN + tempSecond);
						
				if (totalTime > MAX_TOTAL_TIME) {
					System.out.println("Total time exceeds 80 minutes");
					//reset the total time back to it's original value
					totalTime -= tempMinute*SEC_IN_MIN + tempSecond;
					repeat = true;
				}
				else { //totalTime is OK, break out of loop
					repeat = false;
				}		
			} while (repeat);
			
			//Set the Song time to the song after checking, then add
			song.setPlayMinute(tempMinute);
			song.setPlaySecond(tempSecond);
	   		songs.addElement(song);	
	    }
	} 	
	

	/*****************
	 Part of the add method.  Takes in an MyArray object, which is the CD
	 array in this library, and an index which corresponds to the element
	 that has just been added.  It prompts the user to confirm to add this
	 CD (which is already done) or NOT to add it (in this case, it will 
	 remove it)	 
	 
	 @param cds, the MyArray myCDs in this library
	 @param cdArrayIndex, the index of the CD to be confirmed
	******************/
	private static void confirmAdd (MyArray cds, int cdArrayIndex)
		throws IOException {
		
		//the CD that has just been added
		CD addedCD = ((CD)cds.getElementAt(cdArrayIndex));
		
		//make sure that the user only enter y or n
		boolean flag = false;
		do{
			System.out.println("Add new CD? [Y or N]: ");
			String yesNo = stdin.readLine();
			
			//if yes, change is set to true (for save before exit)
			if (yesNo.equalsIgnoreCase("y")) {
				System.out.println("New CD is successfully added!\n");
				flag = false;
				change = true;
			}
			
			//if no, set everything in that CD back to default 	
			else if (yesNo.equalsIgnoreCase("n")) {
				//remove that CD	
				cds.removeElementAt(cdArrayIndex);
				//reset the CD at the end of the array with a default CD 
				cds.setElementAt(new CD (DEFAULT_SIZE), myCDs.getSize());
				//decrement the index for this library
				index--;	
				flag = false;
			}	
			else { 
				System.out.println("Error: Please enter y or n");
				flag = true;
			}	
		}while (flag);			
	
	}
					
	/*****************
	 Sets the CD (and songs) info, print the info out, and confirm the
	 user to add or not
	******************/	
	private static void add() throws IOException {
		//create a temporary CD that stores the CD in the current index
		CD cd = (CD) myCDs.getElementAt(index);

		//sets the CD Info
		setCDInfo (cd);
		
		System.out.println("\nNew CD details:");	
		
		//print the newly added CD
		printCDInfo (cd);
		
		//print its songs info
		printSongsInfo(cd);
		
		//After printing, confirm user to add the CD to the Library or not
		//(index-1) is passed because the index has been incremented after
		//setCDInfo	
		confirmAdd (myCDs, index-1);
	}

	/*****************
	 Returns the index of a particular CD after lookup. If there is more than
	 one match, it will prompt the user to enter the desired index.
	 
	 @param prompt, a string to be used for prompt the user
	******************/
	private static int findIndex (String prompt) throws IOException {
		
		//a boolean array that will stores the results from lookup
		boolean [] tempArray = lookup();
		
		int numMatches = 0;//number of matches found
		int foundIndex = 0;//the desired index
		
		//Count the number of matches (numMatches incremenets if each
		//time a true value is encountered
		for (int i = 0; i < tempArray.length; i++) {
			if (tempArray[i]) {
				numMatches++;
			}
		}
		
		//if there is no matches, return -1
		if (numMatches == 0) {
			foundIndex = -1;
		}
		
		else {
			//there is only one match, make sure that found element contains
			//a true value (just to make sure)
			if (numMatches == 1) {
				for (int i = 0; i < tempArray.length; i++) {
					if (tempArray[i]) {
						foundIndex = i;
					}
				}
			}	
			
			//numMatches > 1, more than one match
			else { 
				
				//repeat if the entered index is either out of bound or
				//the element does not contain a true value (which means
				//that the CD is not the one desired)
				do {
					//must be between 0 and size-1
					foundIndex = readInt(stdin, 0, myCDs.getSize()-1,
						 ("Please enter disc number to be " + prompt + ": "));
					
					if (!tempArray[foundIndex]) {
					System.out.println(
						"This CD does not fit the search criteria.\n");
					}
				
				}while (!tempArray[foundIndex]);
			}
		}	
		
		return foundIndex;
	}				
	
	/*****************
	 Removes the element at the foundIndex
	******************/	
	private static void remove() throws IOException{
		int removeIndex = findIndex("removed");
		
		//if there is no match, don't do anything
		if (removeIndex == -1) {
			return;
		}			
		else {
			//remove the element at that index
			myCDs.removeElementAt(removeIndex);
			//set the element at the end of the array back to default
			myCDs.setElementAt(new CD (DEFAULT_SIZE), myCDs.getSize());
			//decrement the index in this library
			index--;
			System.out.println("Selected CD has been removed.\n");		
		}
		
		change = true;
	}
	
	/*****************
	 Prints the passed CD info by calling its to toString method
	 
	 @param cd, a CD to be printed 
	******************/		
	private static void printCDInfo (CD cd) {
		System.out.println(cd.toString());
	}
	
	/*****************
	 Prints the passed CD's Songs info by calling the each song's toString
	 
	 @param cd, a CD that contains the songs to be printed
	******************/	
	private static void printSongsInfo (CD cd) {
		//create a temporary songArray of that passed CD
		MyArray tempSongsArray = cd.getMySongArray();
		
		//print each song's info    (or cd.getNumSongs)
		for (int i = 0; i < tempSongsArray.getSize(); i++) {
			Song song = (Song) tempSongsArray.getElementAt(i);
			System.out.println("Song " + (i+1) + ": " + song.toString());
		}
	}
	
	/*****************
	 The Modify Artist option of the modify method. Sets the artist name to
	 the passed CD's artist variable, and prints the updated info
	 
	 @param cd, a CD to be modified
	******************/	
	private static void modifyArtist (CD cd) throws IOException {
		cd.setArtist(readString (stdin, "Please enter new Artist Name: "));
		
		System.out.println("");
		printCDInfo(cd);
		System.out.println("");
	}
	
	/*****************
	 The Modify CD Title option of the modify method. Sets the CD title to
	 the passed CD's albumTitle variable, and prints the updated info
	 
	 @param cd, a CD to be modified	 
	******************/	
	private static void modifyCDTitle (CD cd) throws IOException {
		cd.setAlbumTitle(readString (stdin, "Please enter new CD Title: "));
		
		System.out.println("");
		printCDInfo(cd);
		System.out.println("");
	}
	
	/*****************
	 The Modify Release Year option of the modify method. Sets the release
	 year to the passed CD's releaseYr variable, and prints the updated info
	 
	 @param cd, a CD to be modified	 
	******************/	
	private static void modifyReleaseYear (CD cd) throws IOException {
		cd.setReleaseYr(readInt (stdin, 1000, 9999,
			 "Please enter new year: "));
		
		System.out.println("");
		printCDInfo(cd);
		System.out.println("");
	}
	
	/*****************
	 The Modify Song Number option of the modify method. It takes in a song 
	 number and compares it to the number of songs in the passed CD. It adds,
	 removes, or doesn't change songs to the passed CD depending on how
	 the numbers compare
	 
	 @param newNumSong, the new number of songs
	 @param modifycd, the CD to be modified	 
	******************/	
	private static void modifySongNumber (int newNumSong, CD modifycd)
	    throws IOException {
		
		if (newNumSong > modifycd.getNumSongs()) {
			addSongs(newNumSong, modifycd);
		}
		
		else if (newNumSong < modifycd.getNumSongs()) {
			removeSongs(newNumSong, modifycd);
		}
		
		//newNumSong == original number of song
		else {
			System.out.println("There is no change made.\n");
		}
		
		//print the updated info
		printCDInfo (modifycd);
		printSongsInfo (modifycd);
		System.out.println("");
					
	}

	/*****************
	 Part of the modifySongNumber method. It sets the passed number of songs
	 to the passed CD and set each additional song's info by calling the
	 setSongInfo method.
	 
	 @param newNumSong, the new number of songs
	 @param modifycd, the CD to be modified	 
	******************/
	private static void addSongs (int newNumSong, CD modifycd)
		throws IOException {	
		
		//set the new song number
		modifycd.setNumSongs(newNumSong);
		
		//set each song's info; pass the CD, total time of the CD, and the
		//"end index" to setSongInfo method
		setSongInfo (modifycd, modifycd.getTotalTime(), 
			modifycd.getMySongArray().getSize());
	
		System.out.println("");
		
		//calculate total time
		modifycd.getTotalTime();
	}

	/*****************
	 Part of the modifySongNumber method. It sets the passed number of songs
	 to the passed CD and removes the extra ones based on User's choice.
	 An updated list is printed out each time the user removes a song
	 
	 @param newNumSong, the new number of songs
	 @param modifycd, the CD to be modified
	******************/	
	private static void removeSongs(int newNumSong, CD modifycd)
		throws IOException {
		
		//create a temporary array to tosre teh
		MyArray tempSongArray =	modifycd.getMySongArray();
		
		modifycd.setNumSongs(newNumSong);
		
		//this variable keep track of number of songs left after each remove
		int tempSongNumber = tempSongArray.getSize();
		
		//remove extra songs until the number of songs left matches the new
		//number of songs
		for (int i = (tempSongArray.getSize()); i > (newNumSong); i--) {
			
			//prompt the user to input which song to remove
			int removeIndex = (readInt(stdin, 1, tempSongNumber,
			 "Enter song number to remove: ")-1); //actual index is 1 smaller
												  //than the user's input
			//print the deleted song
			System.out.println("Song " + (removeIndex+1) + ": " +
				 ((Song)tempSongArray.getElementAt(removeIndex)) +
				 " has been removed.\n");
			
			tempSongNumber--;
			
			//remove the element from the array and set the last element
			//back to default
			tempSongArray.removeElementAt(removeIndex);
			tempSongArray.setElementAt(new Song(), 
				modifycd.getMySongArray().getSize());
			
			//print the updated list of songs
			printSongsInfo (modifycd);
			
			//calculate total time
			modifycd.getTotalTime();			
			System.out.println("");									
		}
	
	}
	
	/*****************
	 The Modify Song option of the modify method. It prompts the user for 
	 which song to modify and then prompts to enter the new info for that
	 song. It then prints the updated info after setting them
	 
	 @param modify, the CD to be modified
	******************/									
	private static void modifySong (CD modifycd) throws IOException {
		
		//the song to be changed
		int songNumber = readInt (stdin, 1, 
			modifycd.getMySongArray().getSize(),
			"Enter the song number to change: ");
		
		//create a temp song with the correct index (number-1)
		Song tempSong = 
			((Song)modifycd.getMySongArray().getElementAt(songNumber-1));
			
		//prompt user to enter the new details and sets them
		System.out.println("\nEnter song details for song " + (songNumber));
		tempSong.setSongTitle(readString (stdin, "Song name: "));
		
		//varialbles that keep track of entered time			
		int tempMin = 0;
		int tempSec = 0;
		//subtract the times of that song from the total
		int total = modifycd.getTotalTime() - 
			(tempSong.getPlayMinute()*SEC_IN_MIN + tempSong.getPlaySecond());						
		boolean flag = false;
				
		//make sure that the total time does not exceed 80 minutes
		do {
			System.out.println("Enter Time details for new song:");
			tempMin = readInt (stdin, 0, 79, "Minutes: ");
			tempSec = readInt (stdin, 0, 59, "Seconds: ");
			total += tempMin*SEC_IN_MIN + tempSec;
			//exceeds 80 minuetes
			if (total> MAX_TOTAL_TIME) {
				System.out.println("Total time exceeds 80 minutes");
				//sets total back to original value
				total -= (tempMin*SEC_IN_MIN + tempSec);
				flag = true;
			}
			else { //time is OK
				flag = false;
			}		
		} while (flag);
		
		//sets those time to the song and re-calculate total time
		tempSong.setPlayMinute(tempMin);
		tempSong.setPlaySecond(tempSec);						
		modifycd.getTotalTime();
		System.out.println("");
		
		//print the updated CD and songs info
		printCDInfo (modifycd);	
		printSongsInfo (modifycd);
		System.out.println("");		
	
	}		

	/*****************
	 Lookup the CD to be modified. If match is found, print out the CD's info
	 and display the Modify Menu. It then calls the appropriate methods
	 according to user's choice
	******************/					
	private static void modify() throws IOException {
		//the index of the modify CD
		int modifyIndex = findIndex("modified");
		//no match is foud, don't do anything
		if (modifyIndex == -1) {
			return;
		}	
		else{
			//Get the modify CD
			CD modifyCD = ((CD)myCDs.getElementAt(modifyIndex));	 
			//Print that CD's info
			printCDInfo (modifyCD);
			printSongsInfo (modifyCD);
			System.out.println("");
			
			//Go back to the Modify Menu once modifying is done (unless 
			//option 6 is chosen, which will go back to the Main Menu)
			boolean repeat = true;
		
			do{	
				modifyMenu();
				int choice = readInt (stdin, 1, 6, "Enter your choice: ");
				System.out.println("");
				switch (choice) {
					
					//all cases will change the variable "change" to true
					//except case 6
					case 1: 
						modifyArtist(modifyCD);
						change = true;
						break;
									
					case 2: 
						modifyCDTitle(modifyCD);
						change = true;
						break;
					
					case 3:	
						modifyReleaseYear(modifyCD);
						change = true;
						break;
			
					case 4:
						printSongsInfo (modifyCD);
						System.out.println("");
						//get the new number of songs from the user
						int newNumSong = readInt (stdin, 1, MAX_NUM_SONGS,
							"Enter the new song number: ");
						modifySongNumber(newNumSong, modifyCD);
						change = true;							
						break;
		
					case 5: 
						printSongsInfo(modifyCD);
						System.out.println("");
						modifySong(modifyCD);
						change = true;
						break;
					
					case 6: 
						repeat = false;
						break;
					
				}			
					
			} while (repeat); 
		}			
	}
	
	/*****************
	 Prompts the user for an artist name and search the CD Library for 
	 matches. It takes in a boolean array (all false initially) and sets
	 the boolean value in the array to true if the CD is found at that
	 index in the library. It returns the number of matches found
	 (the number of true values in the boolean array)
	 
	 @param b, a boolean array that indicates the matching indices at the
	 		   end of the method (this array is not returned)
 	 @return numMatches, the number of matches	 		   
	******************/		
	private static int lookupByArtist (boolean [] b) throws IOException {
		int numMatches = 0; //stores the number of matches
		
		//prompt user for artist name
		String artistName = readString (stdin, "Please enter artist name: ");
		
		//search the current library; for each match found, set the boolean
		//array's value to true at that index, increment numMatches
		for (int i = 0; i < myCDs.getSize(); i++) {
			if(((CD)myCDs.getElementAt(i)).getArtist().equals(artistName)) {
				b[i] = true; 
				numMatches++; 
			}									
		}
		return numMatches;
	}
	
	/*****************
	 **Must be excuted after the lookupByArtist method**
	 Prints the search results from the lookupByArtist method in the format
	 specified in the requiements. It prints all the CD's that are indicated
	 by a true value in the passed boolean array. The field "Disc Number" is
	 added to the display for user to choose in case there are multiple
	 matches.
	 
	 @param numMatches, the number of matches found in the library
	 @param b, the boolean array that contains the "indications"
	******************/	
	private static void printArtistResults (int numMatches, boolean [] b) {
		int matches = 1; //Match number in the header of the printed results
		
		System.out.println("Number of matches is " + numMatches + "\n");
		for (int i = 0; i < b.length; i++) {
			
			//only prints those that are indicated by a true value
			if (b[i] == true) {
				System.out.println("#### Match " + matches +
				" ####\n" + ((CD)myCDs.getElementAt(i)).toString() +
				"\nDisc Number: " + i + "\n");								
				matches++;
			}
		}		
	}
	
	/*****************
	 Prompts the user for a CD Tile and search the CD Library for 
	 matches. It takes in a boolean array (all false initially) and sets
	 the boolean value in the array to true if the CD is found at that
	 index in the library. It returns the number of matches found
	 (the number of true values in the boolean array)
	 
	 @param b, a boolean array that indicates the matching indices at the
	 		   end of the method (this array is not returned)
 	 @return numMatches, the number of matches	 
	******************/	
	private static int lookupByCDTitle (boolean [] b) throws IOException {
		int numMatches = 0; //stores the number of matches
		
		//prompt user for CD Title
		String CDTitle = readString (stdin, "Please enter CD Title: ");

		//search the current library; for each match found, set the boolean
		//array's value to true at that index, increment numMatches
		for (int i = 0; i < myCDs.getSize(); i++) {
			if(((CD)myCDs.getElementAt(i)).getAlbumTitle().equals(CDTitle)) {
				b[i] = true;
				numMatches++;
			}									
		}
		return numMatches;
	}
	
	/*****************
	 **Must be excuted after the lookupByCDTitle method**
	 Prints the search results from the lookupByCDTitle method in the format
	 specified in the requiements. It prints all the CD's that are indicated
	 by a true value in the passed boolean array. The field "Disc Number" is
	 added to the display for user to choose in case there are multiple
	 matches.
	 
	 @param numMatches, the number of matches found in the library
	 @param b, the boolean array that contains the "indications"	 
	******************/	
	private static void printCDTitleResults (boolean [] b) {
		int matches = 1; //Match number inthe header of the printed results
		
		for (int i = 0; i < b.length; i++) {
			//create a temporary CD
			CD tempCD = ((CD)myCDs.getElementAt(i));
			
			//only prints those that are indicated by a true value
			if (b[i] == true) {
				System.out.println("\n#### Match " + matches +
				" ####\n" + tempCD.toString());								
				
			//print the song info of that CD	
				printSongsInfo(tempCD);
				System.out.println("Disc Number: " + i + "\n");
				matches++;		
			}
		}
	}
	
	/*****************
	 Takes in a song name and search the CD Library for 
	 matches. It takes in a boolean array (all false initially) and sets
	 the boolean value in the array to true if the song is found in the CD 
	 at that index in the library. It returns the number of matches found
	 (the number of true values in the boolean array)
	 
	 @param b, a boolean array that indicates the matching indices at the
	 		   end of the method (this array is not returned)
	 @param songName, the song name to be searched		   
 	 @return numMatches, the number of matches	 
	******************/	
	private static int lookupBySongName (boolean [] b, String songName)
		throws IOException {
		
		int numMatches = 0; //stores the number of matches
 		 		
		for (int i = 0; i < myCDs.getSize(); i++) {
			//create a temporary CD
			CD tempCD = ((CD)myCDs.getElementAt(i));
			
			//search each song
			for (int j = 0; j < tempCD.getMySongArray().getSize(); j++) {
				
				//for each match found, set the boolean array's value to
				//true at that index, increment numMatches				
				if(((Song)tempCD.getMySongArray().
					getElementAt(j)).getSongTitle().equals(songName)) {
					b[i] = true;
					numMatches ++;
		
				}
			}
		}
		return numMatches;
	}
	
	/*****************
	 **Must be excuted after the lookupBySongName method**
	 Prints the search results from the lookupBySongName method in the format
	 specified in the requiements. After locating the CD with the song (true
	 in the passed array), it locate the exact position of the passed 
	 songName in that CD.  It prints all the CD's that are indicated
	 by a true value in the passed boolean array with that song's info.
	 The field "Disc Number" is added to the display for user to choose
	 in case there are multiple matches.
	 
	 @param songName, the song name to be searched
	 @param b, the boolean array that contains the "indications"	 
	******************/	
	private static void printSongNameResults (boolean [] b, String songName) {
		int matches = 1;//value for displaying Match Number
		
		for (int i = 0; i < b.length; i++) {
			//create a temporary CD
			CD tempCD = ((CD)myCDs.getElementAt(i));
			
			//if that CD contains the desired song, print that CD and song
			//info
			if (b[i] == true) {				
				System.out.println("\n#### Match " + matches +
				" ####\n" + ((CD)myCDs.getElementAt(i)).toString());								
				
				//locate that song in this CD
				for (int j = 0; j < tempCD.getMySongArray().getSize(); j++) {
					
					if (((Song)tempCD.getMySongArray().getElementAt(j)).
					getSongTitle().equals(songName)) {
					
					//print that song
					System.out.println("Song: " + ((Song)((CD)myCDs.
					getElementAt(i)).getMySongArray().getElementAt(j)));
					}								
				}
				
				System.out.println("Disc Number: " + i +"\n");
				matches++;
			}	
		}
	}
		
	/*****************
	 prints the no match message (used in lookup)
	******************/
	private static void printNoMatches() {
		System.out.println ("No match is found in the Library.\n");
	}	 							
	
	/*****************
	 Displays the LookUp menu and prompts user for a choice. Calls the 
	 appropriate search mechanism by passing an array of boolean values
	 (all values are set to false). When done, returns this array.
	 
	 @return array, a boolean array that contains the matches
	 				destinated by true values
	******************/			
	private static boolean[] lookup() throws IOException {
	
		lookupMenu();
		int choice = readInt (stdin, 1, 3, "Enter your choice: ");
		System.out.println("");
		//create array of boolean to be returned with CD Library's
		//capacity in case all of the cases are matched
		boolean [] array = new boolean [myCDs.getCapacity()];
		
		//set all values to false
		for (int i = 0 ; i < array.length ; i++){
			array[i]=false;
		}	
		int numMatches = 0;//stores the number of matches
	
		switch (choice) {
				
			case 1: numMatches = lookupByArtist (array);
					if (numMatches == 0) {
						printNoMatches();
					}
					else {
						printArtistResults (numMatches, array);
					}		
					break;
								
			case 2: numMatches = lookupByCDTitle (array);
					if (numMatches == 0) {
						printNoMatches();
					}
					else {
						printCDTitleResults (array);
					}		
					break;	
					
			case 3: //get a song name from the user(pass to lookupBySongName)
					String songName = readString 
						(stdin, "Please enter song name: ");
			
					numMatches = lookupBySongName (array, songName);
					
					if (numMatches == 0) {
						printNoMatches();
					}
					else {
						printSongNameResults (array, songName);
					}
					break;			
		}
			
		return array;
	}			

	/*****************
	 Verifies the source file to be loaded is in the right format. It
	 returns the filename of the checked file
	 
	 @return filename, the filename of the checked source
	******************/	
	private static String verifyLoadFile () throws IOException {
		File srcFile; //File object
		String filename; //stores the filename
				
		boolean repeat = false;
		
		//repeat until a right file name is entered.
		do{
			filename = readString (stdin,
				"Please enter pathname of source file: ");
			
			//create a new file with that filename	
			srcFile = new File(filename); 
			repeat = false;
			
			if (!srcFile.exists()) {
				System.out.println("Error, file not found\n");
				repeat = true;
			}
			else if (!srcFile.isFile() || !srcFile.canRead() ||
				!FileFormatVerifier.verify(filename)) {
				
				System.out.println("Error, file not readable\n");
				repeat = true;
			}
			else 
				System.out.println("");
		
		} while(repeat);
		
		return filename;
	}	
	
	/*****************
	 Loads the source file by tokenizing the string. It sets all the info
	 into a tempCD and add it to the MyArray in this library.
	 **It does not check for time limit at this time!!**
	 **An error will NOT occur if there are more than one "|" separating 
	   the fields
	******************/	
	private static void load() throws IOException{
		CD tempCD;
		MyArray tempSongs;
		File srcFile = new File (verifyLoadFile());//File with valid name
			
		//BufferedReader for reading information from file
		BufferedReader inFile = new BufferedReader(new FileReader(srcFile));
		
		while(inFile.ready()) { //as long as there are more lines to read
			
			//create a CD at the current index
			tempCD = ((CD)myCDs.getElementAt(index));
			//create a MyArray for the song array of this CD
			tempSongs = tempCD.getMySongArray();

			String line = inFile.readLine();
			
			//check for empty lines
			if (line.equals("")) {
				break;
			}	
			
			//create a StringTokenizer object with that line (no delimiters
			//at this time (except default)) Each "nextToken" will have its
			//own delimiters ("|" or ":")
			StringTokenizer t = new StringTokenizer (line);
			
			//set the CD data members with the next token
			tempCD.setArtist(t.nextToken("|"));
			tempCD.setAlbumTitle(t.nextToken("|"));
			tempCD.setReleaseYr(Integer.parseInt(t.nextToken("|")));
			tempCD.setNumSongs(Integer.parseInt(t.nextToken("|")));
			
			//set each song's data member in that CD
			for (int i = 0; i < tempCD.getNumSongs(); i++) {
				//create a temporary Song
				Song tempSong = ((Song)tempSongs.getElementAt(i));
				tempSong.setSongTitle(t.nextToken("|"));
				tempSong.setPlayMinute(Integer.parseInt(t.nextToken(":|")));
				tempSong.setPlaySecond(Integer.parseInt(t.nextToken(":|")));
				//add the tempSong to the tempSongs (the MyArray)
				tempSongs.addElement((Song)tempSongs.getElementAt(i));
			}
			//calculate total time
			tempCD.getTotalTime();
			
			//add the tempCD to the MyArray in this library
			myCDs.addElement(tempCD);
			//increment index
			index++;
		}
		inFile.close();
		change = true;
	}
	
	/*****************
	 Verify that the destination file is writable. If the file already 
	 exists, prompt the user to overwrtie it or not.
	 
	 @return filename, the filename of the destination file in which data
	  		 will be stored
	******************/	
	private static String verifySaveFile() throws IOException {
		File dstFile; 
		String filename;  //stores the file name
		boolean overwrite = false;
		boolean repeat = false;
		//this loop continues if the user does not want to overwrite file
		do {
			
			//this loop continues until the user enter a right filename
			do {
				filename = readString (stdin,
					 "Enter the filename to store to: ");			
				
				dstFile = new File (filename);
				if (dstFile.exists() && (dstFile.isDirectory() || 
					!dstFile.canWrite())) {
					
					System.out.println("Error, file not writable");
					repeat = true;
				}
				else {
					repeat = false;
				}
			} while (repeat);
		
			if (dstFile.exists()) {
				System.out.print("File already exists, overwrite? [Y or N]: ");
				boolean loop = false;
				String yesNo = stdin.readLine();
				
				//this loop makes sure that the user enter y or n
				do {
					if (yesNo.equalsIgnoreCase("y")) {
						loop = false;
						overwrite = false;
					}	
					else if (yesNo.equalsIgnoreCase("n")) {
						loop = false;
						overwrite = true;
					}
					else {
						System.out.print("Error, please enter y or n: ");
							loop = true;	
					}
				} while (loop);
			}
			else { //the file doesn't exist, break out of loop
				break;
			}	
				
		} while (overwrite);
		
		return filename;
	}			
	
	/*****************
	 Saves the data in the library to a file using a PrintWriter object.
	******************/	
	private static void save() throws IOException {
		//create a file with the verified name
		File dstFile = new File (verifySaveFile());

		PrintWriter outFile = new PrintWriter (new BufferedWriter
			(new FileWriter (dstFile)));
		
		//loop through the entire library, print the data members after
		//appending them withappropriate symbols ("|" or ":")
		for (int i = 0; i < myCDs.getSize(); i++) {
			CD tempCD = ((CD)myCDs.getElementAt(i));
			outFile.print(tempCD.getArtist() + "|");
			outFile.print(tempCD.getAlbumTitle() + "|");
			outFile.print(tempCD.getReleaseYr() + "|");
			outFile.print(tempCD.getNumSongs() + "|");
			
			for (int j = 0; j < ((CD)myCDs.getElementAt(i)).
				getMySongArray().getSize(); j++) {
				
				//tempSong for each song in the CD
				Song tempSong = ((Song)((CD)myCDs.getElementAt(i)).
					getMySongArray().getElementAt(j));
					
				outFile.print(tempSong.getSongTitle() + "|");
				outFile.print(tempSong.getPlayMinute() + ":");
				outFile.print(tempSong.getPlaySecond() + "|");
			}
			outFile.print("\n");
		}
		outFile.close();
		change = false;//set change back to false	 	
	}					
			
	/*****************
	 Prints out the main menu
	******************/	
	private static void mainMenu() {
	
		System.out.println ("###########################################");
		System.out.println ("1. Add CD");
		System.out.println ("2. Remove CD");
		System.out.println ("3. Modify CD");
		System.out.println ("4. Lookup CD");
		System.out.println ("5. Load existing database from file");
		System.out.println ("6. Store database to file");
		System.out.println ("7. Exit");
		System.out.println ("###########################################");
		
		
	}
	
	/*****************
	 Prints out the lookup menu
	******************/	
	private static void lookupMenu() {
	
		System.out.println ("Enter search mechanism");
		System.out.println ("1. Lookup by Artist");
		System.out.println ("2. Lookup by CD Title");
		System.out.println ("3. Lookup by Song Name\n");
		
	}
	
	/*****************
	 Prints out the modify menu
	******************/	
	private static void modifyMenu() {
	
		System.out.println ("Enter item to modify");
		System.out.println ("1. Change Artist");
		System.out.println ("2. Change Title");
		System.out.println ("3. Change Year of Release");
		System.out.println ("4. Change Number of Songs");
		System.out.println ("5. Change a Song");
		System.out.println ("6. Exit and Return to Main Menu\n");
		
	}

}//class CDLibrary