/*******************************************************************************
 Main Class:       CDLibrary
 Author:           Chi-Kai (Clement) Chow chikaichow@students.wisc.edu
 Completion Date:  Tuesday, December 14th, 1999

 Course:           CS 302, Fall '99, Section 17
 TA:			   David Schneider
 Assignment:       Project 2

 Compiler:         CodeWarrior (JDK 1.2)
 Platform:         Windows NT 4.0
*******************************************************************************/

/*****************
 Represents an object containing an array of any type of Object; it contains
 methods that can manipulate the array (add, remove, get and set elements)
  
 Bugs: None  			
******************/
class MyArray {

	private Object [] array; //An array of any Object type
	private int capacity; //Maximum size of the array
	private int size; //current size of the array (index)
	
	/*****************
	 Constructor - initializes capacity with the passed value and set size
	 to be 0; Create an array with the capacity value
	 
	 @param initCapacity the initial capacity of the array
	******************/		
	public MyArray (int initCapacity) {
		capacity = initCapacity;
		size = 0;
		array = new Object [capacity];
	}	

	/*****************
	 Constructor - Takes in an array as a parameter and sets it to the
	 private data member array.  Set size = 0
	 
	 @param a an Object array
	******************/	
	public MyArray (Object [] a) {
		array = a;
		capacity = array.length;
		size = 0;
	}
	
	/*****************
	 Constructor - initializes data members by default, with 0 capacity
	******************/		
	public MyArray () {
		this(0); //calls the overloaded constructor
	}	
		
	/*****************
	 Returns the current size of the array
	 
	 @return size the current size of the array
	******************/	
	public int getSize () {
		return size;
	}

	/*****************
	 Returns the capacity of the array
	 
	 @return capacity the capacity of the array
	******************/		
	public int getCapacity () {
		return capacity;
	}

	/*****************
	 Returns the element at the passed index, throws exception if index
	 is out of bound
	 
	 @return element an array at the passed index
	******************/		
	public Object getElementAt (int index) {
	
		if(index >= capacity || index < 0){
			throw new ArrayIndexOutOfBoundsException();
		}
		return array [index];
	}
	
	/*****************
	Sets the passed element at the passed index, throws exception if index
	is out of bound 
	******************/	
	public void setElementAt (Object element, int index) {
	
		if(index >= capacity || index < 0){
			throw new ArrayIndexOutOfBoundsException();
		}
		array [index] = element;
	}	
	
	/*****************
	 Removes the element at the passed index; it does this by shifting the
	 elements in the array and sets the last element to null
	******************/		
	public void removeElementAt (int index) {
	
		for (int i = index; i < size-1; i++) {
			array[i] = array[i+1];
		}
		array[size-1] = null;
		size -= 1;
	}
	
	/*****************
	 Adds the passed element at the end; it increases the capacity 
	 automatically if the size reaches the (capacity-1)
	******************/	
	public void addAnyElement (Object element) {
		array[size] = element;
		size++;
		if (size == (capacity - 1)) {
			increaseCapacity();
		}
	}
	
	/*****************
	 Adds the passed element at the end; this method is specific to the the
	 CDLibrary class in that after increase capacity it will initializes the
	 null elements to CD or Song depending on the type of element passed
	******************/		 
	public void addElement (Object element) {
		array[size] = element;
		size++;
		if (size == (capacity-1)) {
			increaseCapacity();
			for (int i = 0; i < array.length; i++){
				if (array[i] == null) {
					if (element instanceof CD){
						array[i] = new CD (CDLibrary.DEFAULT_SIZE);
					}
					else if (element instanceof Song){
						array[i] = new Song();
					}
				}
			}
		}
	}	
	
	
	
	/*****************
	 Doubles the capacity of the array
	******************/	
	private void increaseCapacity() {
		capacity *=2;
		//create a temperory array with the doubled capacity
		Object [] newArray = new Object[capacity];
		
		//copy the elements from the old to the new
		for (int i = 0; i < array.length; i++) {
			newArray[i] = array[i];
		}
		//old gets new
		array = newArray;
	}
	
	/*****************
	 Returns a string for displaying purposes
	 
	 return s, a string that "represents" the elements of the array
	******************/	
	public String toString() {
		String s = "[";
		for (int i = 0; i < size; i++) {
			if (i == size-1) {
				s += array[i] + "]";
			}
			else {
				s += array[i] + ",";
			}
		}
		return s;
	}			
				
}//class myArray	
			
			
	
		