/******************************* FILE HEADER **********************************
      Main Class File:  CDDatabase.java
      File:             SongArray.java
      
      Author:           Data Fok / data@cs.wisc.edu
      Collaborators:    N/A

      Completion Date:  2000/12/8

      Course:           CS 302, Fall 2000, Section 14
      TA:               Jeff Shergali

      Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
      Platform:         Windows NT 4.0
**************************** 80 columns wide *********************************/

/*****************
 Represents an array of song; it contains methods that can manipulate 
 the array (add, remove, get and set Songs)
  
 Bugs: None  			
******************/
class SongArray {
	
	private int numOfSong = 0; //number of song in the array
	
	private Song[] songList; //an array of songs
	
	/*****************
	 Constructor - Create an array with the capacity value
	 
	 @param num the initial capacity of the array
	******************/
	public SongArray(int num) {
		songList = new Song[num];
	}
	
	/*****************
	 Constructor - Create an array with the capacity = 1 (test use)
	******************/
	public SongArray() {
		songList = new Song[1];
	}
	
	/*****************
	Sets the passed song at the passed index, throws exception if index
	is out of bound 
	******************/	
	private void setSongAt(Song mySong, int index) {
		if ( index >= 0 && index < songList.length) {
			songList[index] = mySong;
		}
		else {
			System.out.println("/n The index is out of the range of " + 
			"the songList");
			throw new ArrayIndexOutOfBoundsException();
		}
	}
	
	/*****************
	Expand the array by a specific number of song
	
	@param num the number of song need to be expand in the array
	******************/	
	public void addNumSong(int num) {
		Song[] tempList = new Song[songList.length+num];
		for (int i = 0; i < songList.length ; i++) {
			tempList[i] = songList[i];
		}
		songList = tempList;
	}
	
	/*****************
	 Doubles the capacity of the array
	******************/
	public void doubleSpace() {
		Song[] tempList = new Song[songList.length*2];
		for (int i = 0; i < songList.length ; i++) {
			tempList[i] = songList[i];
		}
		songList = tempList;
	}
		
	/*****************
	 Reduce the capacity of the array to the num of song it has
	******************/
	public void roundSize() {
		Song[] tempList = new Song[numOfSong];
		for (int i = 0; i < (numOfSong-1) ; i++) {
			tempList[i] = songList[i];
		}
		songList = tempList;
	}
			
	/*****************
	 Adds the passed song at the end;
	******************/
	public void addSong(Song mySong) {
		numOfSong ++;
		setSongAt(mySong, (numOfSong-1) );
	}
		
	/*****************
	 Removes the song at the passed index; it does this by putting
	 the last element to "overwrite" the unwanted Song and sets the 
	 last CD to null.Then round off the size if the array.
	******************/	
	public void removeSongAt(int index) { 
	
		setSongAt(songList[numOfSong-1], index);
		songList[numOfSong-1] = null;
		numOfSong--;		
		if ( numOfSong < getCapacity() ) {
			roundSize();
		}
	}
	
	/*****************
	 Returns the capacity of the array
	 
	 @return capacity the capacity of the array
	******************/
	public int getCapacity() {
		return songList.length;
	}
	
	/*****************
	 Returns the current size of the array
	 
	 @return size the current size of the array 
	              (ie number of song in the array)
	******************/
	public int getSize() {
		return numOfSong;
	}
		
	/*****************
	 Returns the song at the passed index
	 
	 @return song the song in the passed index
	******************/
	public Song getSongAt(int index) {
		return songList[index];
	}
	
	/*****************
	 Clear the whole array, by setting all element to null (test use)
	******************/
	public void clear() {
		for ( int i = 0 ; i < songList.length ; i++) {
			songList[i] = null;
		}
		numOfSong = 0;
	}
	
	/*****************
	 Returns whether the array is an empty array
	 
	 @return boolean the "empty or not" status of the array
	******************/
	public boolean isEmpty() {
		for (int i=0; i<songList.length; i++) 
			if ( songList[i] != null )
				return false;
		return true;
	}
	
	/*****************
	 Returns a string for output to a text file (used in save method in 
	 CDDatabase Class)
	 
	 return tempStr, a string that "represents" the songs in the array
	******************/
	public String toString() {
		String tempStr ="";
		for (int i = 0; i < numOfSong ; i++) {
			if ( songList[i] != null ) {
				tempStr = tempStr + songList[i].toString();
			}
		}
		return tempStr;
	}
	
}