/* ***************************** FILE HEADER **********************************
       Main Class File:  Survivor.java
       File:             Time.java
       
       Author:           Jeffrey Shergalis	shergali@cs.wisc.edu
       Collaborators:    <names of collaborators>

       Completion Date:  10/01/2000

       Course:           CS 302, Fall 2000, Section 
       TA:               Jeff Shergalis

       Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
       Platform:         Windows NT 4.0
 **************************** 80 columns wide *********************************/

/*
	Note: There are several constructors contained in this class.  They are
	not all necessary for use with the Survivor program, but may be useful
	for future programs, and thus have been included in this class.  It is
	generally my style to add extra things now that may be needed later
	so that I don't have to change the implementation to much if I wish
	to reuse the class.  As I said, this is not necessary, but it becomes
	quite helpful later if you wish to reuse the time class and really
	wish it had a constructor that would take a starting time, a minimum
	value and a maximum value, or one that just takes a min and a max and
	sets the starting time to be the minimum value.  

	For the Survivor program you really only needed the default constructor
	Time().  If you have any questions about this solution, please feel
	free to email me.  Also if you find errors in the code, please email
	me.  shergali@cs.wisc.edu
*/

/**
 *
 * The Time class is used to hold a time object which can be iterated
 * in steps of 1, it will roll the time back to its minimum value
 * once it has been iterated past its max value
 *
 * Bugs: none known
 * @author Jeffrey Shergalis shergali@cs.wisc.edu
 *
**/
public class Time
{
	private int timeData;
	private int minTime;
	private int maxTime;
	
	/**
	 *
	 * Constructs a time object with a min value of 0 and a max value of 59
	 * and sets the current time to 0
	 *
	 * @param none
	 *
	**/
	public Time()
	{
		minTime = 0;
		maxTime = 59;
		timeData = minTime;
	}

	/**
	 *
	 * Constructs a time object with a min value of 0 and a max value of 59
	 * and sets the current time to "time"
	 *
	 * @param time the current time for the time object (0 < time < 59)
	 *
	**/
	public Time(int time)
	{
		minTime = 0;
		maxTime = 59;

		// here we check to see if the time given is valid for this
		// instance of the class
		if(isValidTime(time))
		{
			timeData = time;
		}
		else
		{
			// System.out.println sends a single line of text to the
			// console window.  
			System.out.println("The time entered: " + time + " is not valid");
			
			// System.exit(0) will exit the current program
			System.exit(0);
		}
	}
	
	/**
	 *
	 * Constructs a time object with a minimum value of "min" and a maximum
	 * value of "max" and sets the current time to "min"
	 *
	 * @param min the minimum time for the time object ( 0 < min)
	 * @param max the maximim time for the time object ( 0 < min < max)
	 *
	**/
	public Time(int min, int max)
	{
		// we check to see if max and min are > 0 and that max > min
		if((max<0)||(min<0)||(max<=min))
		{
			System.out.println("Error in max and min");
			System.exit(0);
		}
		else
		{
			minTime = min;
			maxTime = max;
			timeData = min;
		}
	}

	/**
	 *
	 * Constructs a time object with a minimum value of "min" and a maximum
	 * value of "max" and sets the current time to "time"
	 *
	 * @param time the current time of the time object ( 0 < min < time < max)
	 * @param min the minimum time for the time object ( 0 < min)
	 * @param max the maximim time for the time object ( 0 < min < max)
	 *
	**/
	public Time(int time, int min, int max)
	{
		if((max<0)||(min<0)||(max<=min))
		{
			System.out.println("Error in max and min");
			System.exit(0);
		}

		minTime = min;
		maxTime = max;

		if(isValidTime(time))
		{
			timeData = time;
		}
		else
		{
			System.out.println("The time entered: " + time + " is not valid");
			System.exit(0);
		}
	}

	/**
	 *
	 * returns the current time of the time object
	 *
	 * @param none
	 *
	 * @return an int representing the current value of the time object
	 *
	**/
	public int getValue()
	{
		return timeData;
	}	

	/**
	 *
	 * returns the maximum value possible for the time object
	 *
	 * @param none
	 *
	 * @return an int representing the maximum value of the time ojbect
	 *
	**/
	public int getMaxValue()
	{
		return maxTime;
	}

	/**
	 *
	 * returns the minimum value possible for the time object
	 *
	 * @param none
	 *
	 * @return an int representing the minimum value of the time ojbect
	 *
	**/
	public int getMinValue()
	{
		return minTime;
	}

	/**
	 *
	 * increments the time object by 1 unit
	 *
	 * @param none
	 *
	 * @return returns true if value flipped from max to min otherwise returns
	 * false
	 *
	**/
	public boolean increment()
	{
		if(timeData >= maxTime)
		{
			timeData = minTime;
			return true;
		}
		else
		{
			timeData++;
			return false;
		}
	}

	/**
	 *
	 * increments the time object by "inc" units
	 *
	 * @param inc the number of units to increase the current time 
	 *	
	 * @return returns true if value flipped from max to min otherwise returns
	 * false
	 *
	**/
	public boolean increment(int inc)
	{
		boolean rolled_over = false;

		// here we check of inc is greater than maxTime, if it is then
		// we will definitely roll over, and we need to see how much
		// more we increment (if inc is 2*maxTime + 10, then we would
		// still need to increment by 10) inc % maxTime gives us this
		// number
		if(inc > maxTime)
		{
			inc = inc % maxTime;
			rolled_over = true;
		}
	
		// if the current time is equal to the maximum time, we need to roll
		// over
		if(timeData == maxTime)
		{
			timeData = minTime + inc;
			rolled_over = true;
		}
		// if by adding inc to timeData we will be greater than maxtime
		// we need to roll over and then increase some amount past
		// min time
		else if((timeData+inc)>maxTime)
		{
			timeData = minTime + ((timeData+inc)-maxTime);
			rolled_over = true;
		}
		// otherwise we can just increment our current time
		else
		{
			timeData += inc;
		}

		return rolled_over;
	}	

	/**
	 *
	 * checks "time" to see if it is a valid time for the time ojbect
	 *
	 * @param time the time to be checked
	 *
	 * @return returns true if "time" is valid otherwise returns false
	 *
	**/
	public boolean isValidTime(int time)
	{
		if((time<0) || (time<minTime) || (time>maxTime))
		{
			return false;
		}
		else
		{
			return true;
		}
	}
}
