/******************************* FILE HEADER **********************************
      Main Class File:  Survivor.java
      File:             Time.java
      
      Author:           Data Fok / data@cs.wisc.edu
      Collaborators:    N/A

      Completion Date:  2000/10/1

      Course:           CS 302, Fall 2000, Section 
      TA:               Jeff Shergali

      Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
      Platform:         Windows NT 4.0
      **************************** 80 columns wide *********************************/

/**
 * Time class works as a rollover count up value. Each time the increment 
 * method is called, 1 is added to the current value, If this 
 * would result in the value going above a maximum value, then the
 * current value is reset to the minimum value.
 * Bugs: Does ungraceful System.exit(0) on all error conditions.
 **/

public class Time {

    private int maxValue; //This maxValue is non-inclusive bound on current
    // (i.e. current should never have this value)
    //range: greater than minValue (and not equal to minValue)

    private int minValue; //This minValue is the lowest legal value on current
    //range: less than maxValue (and not equal to maxValue)

    private int current; //current time value range (minValue to maxValue - 1)
      
    
    /**
         *
         * Constructs a time object with a min value of 0 and a max value of 59
         * and sets the current time to 0
         *
         * @param none
         *
        **/
        public Time()
        {
                minValue = 0;
                maxValue = 59;
                current = minValue;
        }


    
    
    
    /**
     * Constructor sets current, maxValue and minValue if max is greater 
     * than min and current lies between min and max (but not equal to either 
     * min or max. Error conditions stop program.
     * @param c current value of the counter
     * @param maximum max value used as upper bound current for rollover
     * @param minimum min value used as lower bound current for rollover
     **/
    public Time(int c, int max, int min){

        //ensure max is greater than min and c is in the range of max to min
        if( max <= min || c > max || c < min)
            System.exit(0); // just terminate the program
        
        current = c;
        maxValue = max;
        minValue = min;
    }

    /**
     * increment increases current count by 1 and does rollover if this 
     * would give a value greater than the maximum.
     * @param none
     * @return true if the count down caused a rollover to minimum, 
     * otherwise false 
     **/
    public boolean increment() {
        //check if decrement would go below min
        if(current >= maxValue) { //rollover needed
            current = minValue; 
            return true;
        }
        else {  // incrementing will not reach the maximum
            current++;
            return false;
        }
    }

    
    /**
     * getValue is an accessor method for current value
     * @param none
     * @return current value
     **/
    public int getValue(){
        return current;
    }
    
    /**
     * getMaxValue is an accessor method for current value
     * @param none
     * @return max value
     **/
    public int getMaxValue(){
    	return maxValue;
    }
    
    /**
     * getMinValue is an accessor method for current value
     * @param none
     * @return min value
     **/
    public int getMinValue(){
    	return minValue;
    }
    
}