/* ***************************** FILE HEADER **********************************
       Main Class File:  AlarmClockApp.java
       File:             AlarmClockApp.java
       
       Author:           Daniel Tomko
       Email:            dtomko@cs.wisc.edu
       Collaborators:    Used Jeffrey Shergalis' Time class from Lesson4

       Completion Date:  12 October 2000

       Course:           CS 302, Fall 2000
       TA:               Daniel Tomko

       Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
       Platform:         Windows NT 4.0
 **************************** 80 columns wide *********************************/

import java.awt.*;  // to use a reference to a Frame
import lesson5.*;   // GUIAlarmDisplay
import javabook.*;  // InputBox and ListBox

/**
 *
 * This is the main class for a simple alarm clock application. It allows the
 * user to set the alarm time, and then it takes the clock time from the
 * system time (using the java.util.Calendar class). The clock ticks up every
 * second until the alarm time is reached at which point the alarm is fired each
 * secod for the (user specified) duration of the alarm. This class handles the
 * creation of the necessary AlarmClock and GUI classes, and also performs the
 * user interaction for obtaining the alarm time and duration.
 *
 * Bugs: The text in the InputBox is yellow. This is very annnoying. =)
 *
 * @author Daniel Tomko dtomko@cs.wisc.edu
 *
**/

class AlarmClockApp {

  /**
   * Main method for an alarm clock application. Handles creation and
   * initializaition of objects.
   *
   * @param argv Command line arguments.
   */

  public static void main(String[] argv) {

    // create the interface object
    GUIAlarmDisplay gui = new GUIAlarmDisplay();

    AlarmClock alarm = new AlarmClock("Daniel Tomko");

    gui.show();
    gui.update("00:00:00", "", false); // show nothing until set

    setupMenu(alarm, gui);

    boolean alarmed = false; // has the alarm gone off?
    boolean signal  = false;
    do {
      signal = alarm.signalActive(); // should we signal this second?

      gui.update(alarm.toString(), alarm.getMessage(), signal);
      alarm.tick();

      //set alarmed to true if the signal has fired or has previously
      alarmed = alarmed || signal;
    } while (!alarmed || signal);
  }

  /**
   * Displays the setup menu and user interaction.
   *
   * @param alarm Actions to set clock are carried out on this object.
   * @param frame ListBox interface is displayed with respect to this frame.
   */

  private static void setupMenu(AlarmClock alarm, Frame frame) {

    ListBox menu = new ListBox(frame);
    InputBox in = new InputBox(frame);
    in.setForeground(Color.black);     // not yellow

    menu.addItem("Set Alarm Hour");
    menu.addItem("Set Alarm Minute");
    menu.addItem("Set Alarm Second");
    menu.addItem("Set Alarm Duration");
    menu.addItem("Set Alarm Clock");
    menu.addItem("Exit Program");

    int hours =    0,
        minutes =  0,
        seconds =  0,
        duration = 0;

    int choice;
    do {
      choice = menu.getSelectedIndex();
      switch (choice) {
        case 0: // set hour
          hours = getPositiveInt(in, "Enter a positive hour", 23);
          break;
        case 1: // set minute
          minutes = getPositiveInt(in, "Enter a positive minute", 59);
          break;
        case 2: // set seconds
          seconds = getPositiveInt(in, "Enter a positive second", 59);
          break;
        case 3: // set duration
          duration = getPositiveInt(in,
                                    "Enter a positive duration",
                                    Integer.MAX_VALUE);
          break;
        case 4: // set alarm clock
          alarm.setAlarm(hours, minutes, seconds, duration);
          alarm.setClock();
          break;
        case 5: // exit program
          System.exit(0);
        default:
          ; // do nothing
      }
    } while (choice < 4);
  }

  /**
   * Uses an input box to obtain a positive integer from
   * the user. Gives the user 5 chances before exiting.
   *
   * @param in An InputBox object to use for interaction.
   * @param prompt A string to prompt for input.
   * @param max The maximum allowable value for the input.
   *
   * @return The integer entered by the user
   */

  private static int getPositiveInt(InputBox in, String prompt, int max) {
    final int NUM_TRIES = 5;
    int i = 0;         // count the number of tries
    int number = -1;
    
    // this loop is both count and sentinal controlled
    // Count (i>5)
    // Sentinal (number is positive
    do {
      number = in.getInteger(prompt + " (max " + max + ")");
    } while (i++ < (NUM_TRIES-1) && (number < 0 || number > max));
    // the minus 1 thing is wierd, but it avoids an off-by-one error

    // If the user exhausted 5 tries without getting it right,
    // there must be something wrong, so exit.
    if (i == NUM_TRIES && (number < 0 || number > max)) {
    	System.out.println("User error: input out of bounds.");
    	System.exit(1); // exit with error condition
    }

    return number;
  }

} // END CLASS
